########################################################
# CG150.1 Headache: Contrast Level Threshold Analyses
#   - Efficacy
#   - Net benefit (full network)
#   - Net benefit (restricted network)
########################################################

source("../../../Functions/recon_vcov.R")
source("../../../Functions/nma_thresh.R")
source("../../../Functions/thresh_forest.R")
source("../../../Functions/d_ab2i.R")

#### STEP 0 ####
# Read in CODA data
source("./Headaches - read CODA.R")

#############################################
# Thresholds for efficacy decision function #
#############################################

#### STEP 1 ####
# First, we must reconstruct the likelihood covariance matrix. Note that a flat
# prior with precision 0.0001 was used.

# Contrast design matrix is
X <- matrix(ncol=7,byrow=T,
            # d2 d3 d4 d5 d6 d7 d8
            c(1, 0, 0, 0, 0, 0, 0,
              0, 0, 1, 0, 0, 0, 0,
              0, 0, 0, 1, 0, 0, 0,
              0, 0, 0, 0, 1, 0, 0,
              0, 0, 0, 0, 0, 1, 0,
              0, 0, 0, 0, 0, 0, 1,
              0,-1, 0, 0, 1, 0, 0,
              0, 0, 0, 0,-1, 1, 0))

# Reconstruct using NNLS
lik.cov <- recon_vcov(post.cov, prior.prec=.0001, X=X)

# Get indices of contrasts in likelihood in full contrast vector
d.a <- d.b <- vector(length=nrow(X))
for (i in 1:nrow(X)){
  d.a[i] <- ifelse(any(X[i,]==-1),which(X[i,]==-1),0)+1
  d.b[i] <- ifelse(any(X[i,]==1),which(X[i,]==1),0)+1
}

d.i <- d_ab2i(d.a, d.b, K=8)

#### STEP 2 ####
# Now we can perform thresholding at the contrast level.

# Thresholds are
thresh <- nma_thresh(X, post.mean, lik.cov, post.cov, opt.max=FALSE, verb=T)

# Label the contrasts and display using a forest plot, along with 95% posterior
# credible intervals as reported in CG150.1

#contr <- paste0("d[",d.a,d.b,"]")
contr <- paste0(d.b," vs. ",d.a)

contr.mean <- post.meandd[d.i]
CI2.5 <- post.q2.5dd[d.i]
CI97.5 <- post.q97.5dd[d.i]
print(thresh <- as.data.frame(cbind(contr,contr.mean,CI2.5,CI97.5,thresh)), row.names=FALSE)

## COLOUR
pdf("Headaches thresholds - contrast level - efficacy.pdf", width=13, height=4)
with(thresh,
     thresh_forest(means=contr.mean, t.lo=lo, t.hi=hi, kstar.lo=lo.newkstar, kstar.hi=hi.newkstar,
                   CI.lo=CI2.5, CI.hi=CI97.5, label=contr, label.title="Contrast",
                   xlab="Change in headache days per month", xlim=c(-3,2), 
                   refline=0, clinsig=0.5, digits=2, cutoff=30)
)
dev.off()

## B&W
pdf("Headaches thresholds - contrast level - efficacy BW.pdf", width=13, height=4)
with(thresh,
     thresh_forest(means=contr.mean, t.lo=lo, t.hi=hi, kstar.lo=lo.newkstar, kstar.hi=hi.newkstar,
                   CI.lo=CI2.5, CI.hi=CI97.5, label=contr, label.title="Contrast",
                   xlab="Change in headache days per month", xlim=c(-3,2), 
                   refline=0, clinsig=0.5, digits=2, cutoff=30, greyscale=TRUE)
)
dev.off()


################################################
# Thresholds for Net Benefit decision function #
################################################

# Define NB parameters
utilNoMig <- 1
utilMig <- as.numeric(post.utilMig)
r <- as.numeric(post.r)
cost <- c(0, 13.86, 8.33, 116.60, 61.02, 9.39, 21.36, 24.09)
cost_trip <- 0.32
wtp <- 20000

# Derive adjustment to posterior mean for NB
A <- (6*(utilNoMig-(((22/24)*(r*utilNoMig+(1-r)*(-utilMig)))
                    +((2/24)*(-utilMig)))))/365 * wtp + cost_trip*6

c_adj <- cost[-1] / A
      
# Altered posterior means for net benefit thresholding are
post.mean.nbadj <- post.mean + c_adj

# Thresholds
thresh.nb <- nma_thresh(X, post.mean.nbadj, lik.cov, post.cov, opt.max=FALSE)

print(thresh.nb <- as.data.frame(cbind(contr,contr.mean,CI2.5,CI97.5,thresh.nb)), row.names=FALSE)


## COLOUR
pdf("Headaches thresholds - contrast level - net benefit.pdf", width=13, height=4)
with(thresh.nb,
     thresh_forest(means=contr.mean, t.lo=lo, t.hi=hi, kstar.lo=lo.newkstar, kstar.hi=hi.newkstar,
                   CI.lo=CI2.5, CI.hi=CI97.5, label=contr, label.title="Contrast",
                   xlab="Change in headache days per month", xlim=c(-3,2), 
                   refline=0, clinsig=0.5, digits=2, cutoff=30)
)
dev.off()

## B&W
pdf("Headaches thresholds - contrast level - net benefit BW.pdf", width=13, height=4)
with(thresh.nb,
     thresh_forest(means=contr.mean, t.lo=lo, t.hi=hi, kstar.lo=lo.newkstar, kstar.hi=hi.newkstar,
                   CI.lo=CI2.5, CI.hi=CI97.5, label=contr, label.title="Contrast",
                   xlab="Change in headache days per month", xlim=c(-3,2), 
                   refline=0, clinsig=0.5, digits=2, cutoff=30, greyscale=TRUE)
)
dev.off()


### Restricted network for NB analysis ###

# We are only concerned with treatments 1,3,6,7 for the CEA,
# so only use those studies and contrasts
ind.NBtrt <- c(3,6,7)-1
ind.NBtrt.contr <- c(4,5,7,8)
X2 <- X[ind.NBtrt.contr,ind.NBtrt]
lik.cov2 <- lik.cov[ind.NBtrt.contr,ind.NBtrt.contr]
post.cov2 <- post.cov[ind.NBtrt,ind.NBtrt]
cost2 <- c(0,8.33,9.39,21.36)

c_adj2 <- cost2[-1] / A

post.mean.nbadj2 <- post.mean[ind.NBtrt] + c_adj2

thresh.nb2 <- nma_thresh(X2, post.mean.nbadj2, lik.cov2, post.cov2, 
                         trt.code=c(1,3,6,7), opt.max=FALSE)

print(thresh.nb2 <- cbind(cbind.data.frame(contr,contr.mean,CI2.5,CI97.5)[ind.NBtrt.contr,]
                          ,thresh.nb2), row.names=FALSE)

### Same results using a subset of contrasts (other contrasts not in CEA network don't really add much) ###
thresh.nb3 <- nma_thresh(X, post.mean.nbadj, lik.cov, post.cov,  
                         trt.sub=c(1,3,6,7), opt.max=FALSE)

print(thresh.nb3 <- cbind(cbind.data.frame(contr,contr.mean,CI2.5,CI97.5)
                          ,thresh.nb3), row.names=FALSE)


## COLOUR
pdf("Headaches thresholds - contrast level - net benefit - RESTRICTED.pdf", width=13, height=3)
with(thresh.nb2,
     thresh_forest(means=contr.mean, t.lo=lo, t.hi=hi, kstar.lo=lo.newkstar, kstar.hi=hi.newkstar,
                   CI.lo=CI2.5, CI.hi=CI97.5, label=contr, label.title="Contrast",
                   xlab="Change in headache days per month", xlim=c(-3,2),
                   refline=0, clinsig=0.5, digits=2, cutoff=30)
)
dev.off()

## B&W
pdf("Headaches thresholds - contrast level - net benefit - RESTRICTED BW.pdf", width=13, height=3)
with(thresh.nb2,
     thresh_forest(means=contr.mean, t.lo=lo, t.hi=hi, kstar.lo=lo.newkstar, kstar.hi=hi.newkstar,
                   CI.lo=CI2.5, CI.hi=CI97.5, label=contr, label.title="Contrast",
                   xlab="Change in headache days per month", xlim=c(-3,2),
                   refline=0, clinsig=0.5, digits=2, cutoff=30, greyscale=TRUE)
)
dev.off()