################################################################################
# CG150.1 Headache: Study Level Threshold Analyses
#   - Efficacy
#   - Net benefit (full network)
#   - Net benefit (restricted network)
################################################################################

require(coda)
source("../../../Functions/nma_threshRE.R")
source("../../../Functions/thresh_forest.R")

#### INITIAL SETUP AND DATA MANIPULATION ####
# - Read in data, arm and trial level separately.
# - Transpose rows to columns for arm level data (not necessary for trial level,
#   only have 2-arm trials)
# - Concatenate into one dataset
# - Read in posterior covariance matrix from WinBUGS

# Arm level
dat.arm <- read.table("./Headaches data ARM LEVEL.txt", head=T, stringsAsFactors=F)
dat.arm$studyID <- 1:nrow(dat.arm) # study ID
dat.arm$b <- dat.arm$t.1 # baseline treatment

# Trial level
dat.trial <- read.table("./Headaches data TRIAL LEVEL.txt", head=T, stringsAsFactors=F)
dat.trial$studyID <- nrow(dat.arm) + 1:nrow(dat.trial) # study ID
dat.trial <- with(dat.trial, 
                  data.frame(studyID, textID, b=t.1, t=t.2, y=y.2, se=se.2, 
                             n_arms, armlvl=FALSE, arm=NA, bl=FALSE)
                  )


## Transpose wide into long data (one row per arm)
dat.arml <- reshape(dat.arm, varying=c("t.1","y.1","se.1","t.2","y.2","se.2",
                                       "t.3","y.3","se.3","t.4","y.4","se.4"), 
                    timevar="arm", idvar="studyID", direction="long")

# Sort data by study and contrast, removing NA rows
dat.arml <- dat.arml[order(dat.arml$studyID, dat.arml$arm, dat.arml$y, na.last=NA),]

# Flag for baseline rows
dat.arml$bl <- rep(FALSE,nrow(dat.arml))
dat.arml$bl[1] <- TRUE
dat.arml$bl[cumsum(table(dat.arml$studyID))[-nrow(dat.arm)]+1] <- TRUE

# Flag for arm level data rows
dat.arml$armlvl <- TRUE

## Concatenate into one dataset
## NOTE: Arm level rows come first then trial level, just like in the
## WinBUGS code, so that e.g. delta parameters are in correct order.
dat <- rbind(dat.arml, dat.trial)


## Read in posterior details from WinBUGS
source("./Headaches - read CODA.R")


#### THRESHOLDS FOR EFFICACY ####
# - Construct necessary matrices
# - Perform thresholding
# - Present results

N <- nrow(dat)                        # number of data points
n1 <- nrow(dat.arm)                   # number of arm level studies
n2 <- nrow(dat.trial)                 # number of trial level studies
n <- n1 + n2                          # total number of studies
K <- length(unique(c(dat$t, dat$b)))  # number of treatments

# Likelihood covariance matrix V. This is simply diagonal as arms are
# independent in arm level data and we only have 2-arm trials for trial level
# data.
V <- diag(dat$se^2)

# Baseline parameter mu design matrix M. Remember that only the arm level data
# points have a mu!
M <- matrix(0, nrow=N, ncol=n1)
for (i in 1:N){
  if (dat$armlvl[i]) M[i,dat$studyID[i]] <- 1
}

# Random effect delta parameter design matrix L. Baseline rows from arm level
# data don't have a delta, all rows from trial level data have a delta.
L <- matrix(0, nrow=N, ncol=N-n1)
j <- 1
for (i in 1:N) {
  if (!dat$bl[i]) {
    L[i,j] <- 1
    j<-j+1
  }
}


## Perform thresholding
thresh <- nma_threshRE(mean.dk = post.mean, lhood = V, post = post.cov.full, 
                       mu.design = M, delta.design = L, opt.max = FALSE)

thresh <- cbind(dat, thresh)

# Make nice row labels
thresh$lab <- ""
for (i in 1:N) {
  if (dat$armlvl[i]) thresh$lab[i] <- paste0(dat$textID[i]," (",dat$t[i],")")
  else thresh$lab[i] <- paste0(dat$textID[i],"\n(",dat$t[i]," vs. ",dat$b[i],")")
}

# Confidence intervals about study estimates
thresh$CI.lo <- thresh$y + qnorm(0.025)*thresh$se
thresh$CI.hi <- thresh$y + qnorm(0.975)*thresh$se


## Create threshold forest plots
# COLOUR
pdf("Headaches thresholds - study level - efficacy.pdf", width=15, height=15)
with(thresh,
     thresh_forest(means=y, t.lo=lo, t.hi=hi, kstar.lo=lo.newkstar, kstar.hi=hi.newkstar,
                   CI.lo=CI.lo, CI.hi=CI.hi, label=lab, label.title="Study (Trt.)",
                   CI.title="95% Confidence Interval",
                   xlab="Change in headache days per month", xlim=c(-8,2), 
                   refline=0, clinsig=0.5, digits=2, cutoff=30, xaxp=c(-8,2,10))
)
dev.off()

# B&W
pdf("Headaches thresholds - study level - efficacy BW.pdf", width=15, height=15)
with(thresh,
     thresh_forest(means=y, t.lo=lo, t.hi=hi, kstar.lo=lo.newkstar, kstar.hi=hi.newkstar,
                   CI.lo=CI.lo, CI.hi=CI.hi, label=lab, label.title="Study (Trt.)",
                   CI.title="95% Confidence Interval",
                   xlab="Change in headache days per month", xlim=c(-8,2), 
                   refline=0, clinsig=0.5, digits=2, cutoff=30, greyscale=TRUE, xaxp=c(-8,2,10))
)
dev.off()


#### THRESHOLDS FOR NET BENEFIT ####
# - Derive linear transformation to NB from efficacy
# - Perform thresholding
# - Present results

# Define NB parameters
utilNoMig <- 1
utilMig <- as.numeric(post.utilMig)
r <- as.numeric(post.r)
cost <- c(0, 13.86, 8.33, 116.60, 61.02, 9.39, 21.36, 24.09)
cost_trip <- 0.32
wtp <- 20000

# Derive adjustment to posterior mean for NB
A <- (6*(utilNoMig-(((22/24)*(r*utilNoMig+(1-r)*(-utilMig)))
                    +((2/24)*(-utilMig)))))/365 * wtp + cost_trip*6

c_adj <- cost[-1] / A

# Altered posterior means for net benefit thresholding are
post.mean.nbadj <- post.mean + c_adj

## Perform thresholding

thresh.nb <- nma_threshRE(mean.dk = post.mean.nbadj, lhood = V, post = post.cov.full, 
                       mu.design = M, delta.design = L, opt.max = FALSE)

thresh.nb <- cbind(dat, thresh.nb)

# Nice row labels
thresh.nb$lab <- thresh$lab

# Confidence intervals about study estimates
thresh.nb$CI.lo <- thresh$CI.lo
thresh.nb$CI.hi <- thresh$CI.hi


## Create threshold forest plots
# COLOUR
pdf("Headaches thresholds - study level - net benefit.pdf", width=15, height=15)
with(thresh.nb,
     thresh_forest(means=y, t.lo=lo, t.hi=hi, kstar.lo=lo.newkstar, kstar.hi=hi.newkstar,
                   CI.lo=CI.lo, CI.hi=CI.hi, label=lab, label.title="Study (Trt.)",
                   CI.title="95% Confidence Interval",
                   xlab="Change in headache days per month", xlim=c(-8,2), 
                   refline=0, clinsig=0.5, digits=2, cutoff=30, xaxp=c(-8,2,10))
)
dev.off()

# B&W
pdf("Headaches thresholds - study level - net benefit BW.pdf", width=15, height=15)
with(thresh.nb,
     thresh_forest(means=y, t.lo=lo, t.hi=hi, kstar.lo=lo.newkstar, kstar.hi=hi.newkstar,
                   CI.lo=CI.lo, CI.hi=CI.hi, label=lab, label.title="Study (Trt.)",
                   CI.title="95% Confidence Interval",
                   xlab="Change in headache days per month", xlim=c(-8,2), 
                   refline=0, clinsig=0.5, digits=2, cutoff=30, greyscale=TRUE, xaxp=c(-8,2,10))
)
dev.off()

## Thresholds for restricted decision
thresh.nb.res <- nma_threshRE(mean.dk = post.mean.nbadj, lhood = V, post = post.cov.full, 
                          mu.design = M, delta.design = L, opt.max = FALSE, trt.sub=c(1,3,6,7))

thresh.nb.res <- cbind(dat, thresh.nb.res)
thresh.nb.res$lab <- thresh$lab
thresh.nb.res$CI.lo <- thresh$CI.lo
thresh.nb.res$CI.hi <- thresh$CI.hi

# Only show thresholds for studies in the restricted network
res.studyID <- c(3,4,5,6,7,8,11)

pdf("Headaches thresholds - study level - net benefit - RESTRICTED.pdf", width=15, height=10)
with(thresh.nb.res[thresh.nb.res$studyID %in% res.studyID,],
     thresh_forest(means=y, t.lo=lo, t.hi=hi, kstar.lo=lo.newkstar, kstar.hi=hi.newkstar,
                   CI.lo=CI.lo, CI.hi=CI.hi, label=lab, label.title="Study (Trt.)",
                   CI.title="95% Confidence Interval",
                   xlab="Change in headache days per month", xlim=c(-8,2), 
                   refline=0, clinsig=0.5, digits=2, cutoff=30, xaxp=c(-8,2,10))
)
dev.off()


# How extreme are the threshold endpoints?

pnorm(thresh.nb$lo, thresh.nb$y, thresh.nb$se)
pnorm(thresh.nb$hi, thresh.nb$y, thresh.nb$se)
