############################################################
# CG159 Social Anxiety: Contrast Level CEA Thresholds
#   - Using logistic normal distribution
############################################################

source("../../../Functions/thresh_forest.R")

##### STEP 1 #####
# Read CODA data to get posterior info
source("./Social Anxiety - read CODA.R")

#### STEP 2 ####
## Net benefit parameters

# WTP
lambda <- 20000

# Utilities (expected value)
U.socanx <- 0.659
U.well <- 0.866

# Costs (expected value)
C.NHS.socanx <- 1037
C.NHS.well <- 645

costs <- read.csv("../costs.csv")
readorder <- costs$k # grab the ordering used in the CEA for easier comparison later

costs <- costs[order(costs$k),]

# Which treatments actually went in to CEA?
ceatrt <- costs$k

## Expected Conditional Net Benefits
attach(costs)

NB.recover <- prob.relapse * ((U.socanx + 0.5*(U.well - U.socanx))*lambda - 
                                (C.trt26 + C.GP26 + 0.5*(C.NHS.socanx + C.NHS.well))) +
              (1-prob.relapse) * (U.well * lambda - (C.trt26 + C.GP26 + C.NHS.well)) -
              (C.trt12 + C.GP12)

NB.norecover <- U.socanx * lambda - C.NHS.socanx - C.trt12 - C.GP12

detach(costs)

# Beta parameter for LOR = - beta * pi/sqrt(3) * SMD
beta.mean <- 1.154
beta.sd <- 0.1008

# Mean effect parameter for deriving P(response)
meanA <- -2.629
sdA <- sqrt(1/0.81)

##### STEP 3 #####
## Derive net benefits

# Find mean of logitnormal distribution, using code from logitnorm::momentsLogitnorm
# but leaving out the second calculation for the variance (we don't need that)
meanLogitnorm <- function(mu, sigma, abs.tol = 0, subdivisions = 100L, ...)
{
  fExp <- function(x) plogis(x) * dnorm(x, mean = mu, sd = sigma)
  integrate(fExp, -Inf, Inf, abs.tol = abs.tol, subdivisions = subdivisions,...)$value
}

# SD of product of two independent RVs
sdprod <- function(sdx, Ex, sdy, Ey){
  sqrt(sdx^2 * sdy^2  + sdx^2 * Ey^2  + sdy^2 * Ex^2)
}

## Expected Net Benefit calculation

cal.mean <- - pi/sqrt(3) * beta.mean
cal.sd <- beta.sd * pi/sqrt(3)

# Compare LOR of response (gCal) with WinBUGS output
gCal <- data.frame(mean= cal.mean * post.mean[ceatrt[-1]-1],
                   sd= sdprod(cal.sd, cal.mean,
                              post.summary$stat[ceatrt[-1]-1,"SD"],
                              post.mean[ceatrt[-1]-1]))
cbind(ceatrt[-1], gCal)

# Compare recovery probabilities (T) with WinBUGS output
Tprob <- mapply(logitnorm::momentsLogitnorm,
            mu=meanA + c(0,gCal$mean), sigma=sqrt(sdA^2 + c(0,gCal$sd)^2))
cbind(ceatrt, Tprob["mean",], sqrt(Tprob["var",]))

.evalNB <- function(badj=0, infl=0, mean.d, sd.d, NB.norec, NB.rec){
  
  prob.rec <- meanLogitnorm(meanA + cal.mean * (mean.d +badj*infl), 
                            sqrt(sdA^2 + 
                                 sdprod(cal.sd, cal.mean, sd.d, (mean.d +badj*infl))^2))
  
  prob.rec * NB.rec + (1-prob.rec) * NB.norec
}

evalNB <- function(badj=0, infl=0, mean.d, sd.d, NB.norec, NB.rec){
  sapply(badj,
         function(x) mapply(.evalNB,
                            mean.d=mean.d, sd.d=sd.d,
                            NB.norec=NB.norec, NB.rec=NB.rec,
                            infl=infl,
                            MoreArgs=list(badj=x)))
}

NB0 <- evalNB(mean.d=c(0,post.mean)[ceatrt], sd.d=c(0,post.summary$stat[,"SD"])[ceatrt],
             NB.norec=NB.norecover, NB.rec=NB.recover)

# Optimal treatment k* using Net Benefit is
kstar <- order(NB0,decreasing=TRUE)[1]

########################################################################################
## Below here are calculations specific to the contrast- and study-level analyses

##### CONTRAST LEVEL ANALYSIS #####
## Get influence matrix

source("../../../Functions/recon_vcov.R")
source("../../../Functions/d_ab2i.R")

# First, we must reconstruct the likelihood covariance matrix.

# Create contrast design matrix X. We'll use the treatment details from the study 
# level data for convenience.
trt.dat <- read.table(file = "../WinBUGS/study data.txt", header = TRUE, sep = "\t")[,1:6]
K <- with(trt.dat, length(unique(c(t.1, t.2, t.3, t.4, t.5)))) -1  # Number of treatments , -1 for NA

# work out which contrasts have data
contr.ab <- data.frame(a=c(), b=c())

for(i in 1:nrow(trt.dat)){
  rowi <- trt.dat[i,which(!is.na(trt.dat[i,2:6]))+1] # non NA elements of ith row
  
  # get contrast from all combinations of treatments
  trtcomb <- combn(rowi, 2, function(x) sapply(x, as.numeric))
  
  a <- apply(trtcomb, 2, min)
  b <- apply(trtcomb, 2, max)
  
  # remove contrasts of treatments against themselves
  iseq <- a==b
  a <- a[!iseq]
  b <- b[!iseq]
  
  if (!all(iseq)) contr.ab <- rbind(contr.ab, cbind(a,b))
}

contr.ab <- unique(contr.ab[order(contr.ab$a, contr.ab$b),])


# Contrast design matrix
X <- matrix(0, nrow=nrow(contr.ab), ncol=K-1)
for (i in 1:nrow(X)){
  if (contr.ab[i,"a"] > 1) X[i, contr.ab[i,"a"] - 1]  <- -1
  if (contr.ab[i,"b"] > 1)   X[i, contr.ab[i,"b"] - 1]    <- 1
}


# Reconstruct using NNLS
# Class model is used, so use the prior mean precision from the gamma distribution
prior.prec <- rep(3.9/0.35, 40)

# Other than for treatment 3
prior.prec[2] <- 0.0001

lik.cov <- recon_vcov(post.cov, prior.vcov=diag(1/prior.prec), X=X)

# Some variances are estimated to be infinite, so remove these contrasts for thresholding
inftf <- is.infinite(diag(lik.cov))
X.noninf <- X[!inftf,]
lik.cov.noninf <- diag(diag(lik.cov)[!inftf])

# Get indices of contrasts in likelihood in full contrast vector
d.a <- d.b <- vector(length=nrow(X))
for (i in 1:nrow(X)){
  d.a[i] <- ifelse(any(X[i,]==-1),which(X[i,]==-1),0)+1
  d.b[i] <- ifelse(any(X[i,]==1),which(X[i,]==1),0)+1
}

d.i <- d_ab2i(d.a, d.b, K=K)

# Now we can derive the influence matrix for the FE model
inflmat <- post.cov %*% crossprod(X.noninf,solve(lik.cov.noninf))

# Add in column(s) of zeros for likelihood contrast(s) with zero influence
tempmat <- matrix(0, nrow=nrow(inflmat), ncol=nrow(contr.ab))
tempmat[,!inftf] <- inflmat
inflmat <- tempmat

## Plot NBs for a range of bias adjustments for a data point
id <- 4 # Pick a data point
badj <- -200:200/10
NBplot <- evalNB(badj=badj, infl=c(0,inflmat[ceatrt[-1]-1,id]),
                 mean.d=c(0,post.mean)[ceatrt], sd.d=c(0,post.summary$stat[,"SD"])[ceatrt],
                 NB.norec=NB.norecover, NB.rec=NB.recover)

matplot(badj, t(NBplot), type="l", 
        ylab="Net Benefit", xlab=paste("Bias Adjustment to data point ",id))

## Create threshold matrix by finding zeros of the NB contrasts involving k*
require(rootSolve)

# Evaluate net benefit contrasts
# Add dummy parameter extendInt to allow this to be passed via uniroot.all to uniroot.
evalNBcontr <- function(badj, row, col, extendInt=NULL){
  trt <- row + (row>=kstar)*1

  return(
    evalNB(badj=badj, infl=rbind(0,inflmat[ceatrt[-1]-1,])[row,col],
           mean.d=c(0,post.mean)[ceatrt[trt]], 
           sd.d=c(0,post.summary$stat[,"SD"])[ceatrt[trt]],
           NB.norec=NB.norecover[trt], NB.rec=NB.recover[trt]) -
    evalNB(badj=badj, infl=rbind(0,inflmat[ceatrt[-1]-1,])[row,col],
           mean.d=c(0,post.mean)[ceatrt[kstar]], 
           sd.d=c(0,post.summary$stat[,"SD"])[ceatrt[kstar]],
           NB.norec=NB.norecover[kstar], NB.rec=NB.recover[kstar])
  )
}


absmin <- function(x) ifelse(identical(x,numeric(0)), NA, x[which.min(abs(x))])

# Threshold array, one matrix for negative thresholds and one for positive
threshmat <- array(NA, dim=c(K-1, nrow(contr.ab), 2))
for(i in 1:(length(ceatrt)-1)){ # row i for treatment
  for(j in 1:nrow(contr.ab)){ # col j for data point
    rts <- uniroot.all(evalNBcontr, interval=c(-20,20), row=i, col=j, 
                       extendInt="yes")

    if(any(rts<0)) threshmat[i,j,1] <- max(rts[rts<0])
    if(any(rts>0)) threshmat[i,j,2] <- min(rts[rts>0])
  }
}

# Plot NB contrast against bias adjustment for a given data point
matplot(badj, t(evalNBcontr(badj, row=1:(length(ceatrt)-1), col=id)), type="l",
        ylab="NB contrast against k*", xlab=paste("Bias Adjustment to data point ",id))
abline(h=0, v=0)
points(x=c(threshmat[,id,]), y=rep(0,(K-1)*2))

# Return the bias invariant interval for each data point
get.int <- function(x, trt.code){
  if (all(is.na(x))){ 
    hi <- Inf
    lo <- -Inf
    hi.newkstar <- lo.newkstar <- NA
  }
  else if (all(x[!is.na(x)]>0)){
    hi <- min(x[!is.na(x)])
    hi.newkstar <- trt.code[which(x[,2]==hi) + (which(x[,2]==hi) >= kstar)*1]
    lo <- -Inf
    lo.newkstar <- NA
  }
  else if (all(x[!is.na(x)]<0)){
    hi <- Inf
    hi.newkstar <- NA
    lo <- max(x[!is.na(x)])
    lo.newkstar <- trt.code[which(x[,1]==lo) + (which(x[,1]==lo) >= kstar)*1]
  }
  else {
    hi <- min(x[x>0 & !is.na(x)])
    hi.newkstar <- trt.code[which(x[,2]==hi) + (which(x[,2]==hi) >= kstar)*1]
    lo <- max(x[x<0 & !is.na(x)])
    lo.newkstar <- trt.code[which(x[,1]==lo) + (which(x[,1]==lo) >= kstar)*1]
  }
  return(data.frame(lo=lo,lo.newkstar=lo.newkstar,hi=hi,hi.newkstar=hi.newkstar))
}

thresh <- as.data.frame(do.call(rbind, apply(threshmat, 2, get.int, trt.code=ceatrt)))

# Label the contrasts and display using a forest plot, along with 95% posterior
# credible intervals as reported in CG150.1

contr <- paste0(d.b," vs. ",d.a)

contr.mean <- dd.mean[d.i]
CI2.5 <- dd.summary$quantiles[d.i,"2.5%"]
CI97.5 <- dd.summary$quantiles[d.i,"97.5%"]
print(thresh <- as.data.frame(cbind(contr,contr.mean,CI2.5,CI97.5,thresh)), row.names=FALSE)

## COLOUR
pdf("Social Anxiety thresholds - contrast level - net benefit.pdf", width=13, height=16)
with(thresh,
     thresh_forest(means=contr.mean, t.lo=lo, t.hi=hi, kstar.lo=lo.newkstar, kstar.hi=hi.newkstar,
                   CI.lo=CI2.5, CI.hi=CI97.5, label=contr, label.title="Contrast",
                   xlab="Standardised Mean Difference", xlim=c(-4,3), 
                   refline=0, digits=2)
)
dev.off()

# Show only contrasts with at least one non infinite threshold
thresh.cut <- thresh[thresh$lo > -Inf | thresh$hi < Inf,]

pdf("Social Anxiety thresholds - contrast level - net benefit CUT.pdf", width=13, height=10)
with(thresh.cut,
     thresh_forest(means=contr.mean, t.lo=lo, t.hi=hi, kstar.lo=lo.newkstar, kstar.hi=hi.newkstar,
                   CI.lo=CI2.5, CI.hi=CI97.5, label=contr, label.title="Contrast",
                   xlab="Standardised Mean Difference", xlim=c(-4,3), 
                   refline=0, digits=2)
)
dev.off()

