############################################################
# CG159 Social Anxiety: Contrast Level Threshold Analysis
# for drug bias, psych bias, and two-way interaction
############################################################

source("../../../Functions/recon_vcov.R")
source("../../../Functions/d_ab2i.R")

#### STEP 0 ####
# Read CODA data
source("./Social Anxiety - read CODA.R")

#### STEP 1 ####
# First, we must reconstruct the likelihood covariance matrix.

# Create contrast design matrix X. We'll use the treatment details from the study 
# level data for convenience.

trt.dat <- read.table(file = "../WinBUGS/study data.txt", header = TRUE, sep = "\t")[,1:6]
K <- with(trt.dat, length(unique(c(t.1, t.2, t.3, t.4, t.5)))) -1  # Number of treatments , -1 for NA

# work out which contrasts have data
contr.ab <- data.frame(a=c(), b=c())

for(i in 1:nrow(trt.dat)){
  rowi <- trt.dat[i,which(!is.na(trt.dat[i,2:6]))+1] # non NA elements of ith row
  
  # get contrast from all combinations of treatments
  trtcomb <- combn(rowi, 2, function(x) sapply(x, as.numeric))
  
  a <- apply(trtcomb, 2, min)
  b <- apply(trtcomb, 2, max)
  
  # remove contrasts of treatments against themselves
  iseq <- a==b
  a <- a[!iseq]
  b <- b[!iseq]
  
  if (!all(iseq)) contr.ab <- rbind(contr.ab, cbind(a,b))
}

contr.ab <- unique(contr.ab[order(contr.ab$a, contr.ab$b),])


# Contrast design matrix
X <- matrix(0, nrow=nrow(contr.ab), ncol=K-1)
for (i in 1:nrow(X)){
  if (contr.ab[i,"a"] > 1) X[i, contr.ab[i,"a"] - 1]  <- -1
  if (contr.ab[i,"b"] > 1)   X[i, contr.ab[i,"b"] - 1]    <- 1
}


# Reconstruct likelihood covariance matrix using NNLS
# Class model is used, so use the prior mean precision from the gamma distribution
prior.prec <- rep(3.9/0.35, 40)
# Other than for treatment 3
prior.prec[2] <- 0.0001

lik.cov <- recon_vcov(post.cov, prior.vcov=diag(1/prior.prec), X=X)

# Some variances are estimated to be infinite, so remove these contrasts for thresholding
inftf <- is.infinite(diag(lik.cov))
X.noninf <- X[!inftf,]
lik.cov.noninf <- diag(diag(lik.cov)[!inftf])

# Get indices of contrasts in likelihood in full contrast vector
d.a <- d.b <- vector(length=nrow(X))
for (i in 1:nrow(X)){
  d.a[i] <- ifelse(any(X[i,]==-1),which(X[i,]==-1),0)+1
  d.b[i] <- ifelse(any(X[i,]==1),which(X[i,]==1),0)+1
}

d.i <- d_ab2i(d.a, d.b, K=K)

#### STEP 2 ####
# Now we can perform thresholding at the contrast level.
# We'll do this manually to derive thresholds for
#    - All drug trts biased by the same amount compared to inactive
#    - All psych trts biased by the same amount compared to inactive
#    - Interaction of two group biases in 2D

# Create contrast "design" matrix
D <- matrix(nrow=K*(K-1)/2, ncol=K-1)
j <- 1
for (i in 1:(K-1)){
  if (i==1) D[j:(j+K-i-1),] <- diag(nrow=K-i)
  else if (i==2) D[j:(j+K-i-1),] <- cbind(rep.int(-1,K-i),diag(nrow=K-i))
  else D[j:(j+K-i-1),] <- cbind(matrix(rep.int(0,(K-i)*(i-2)),ncol=i-2),rep.int(-1,K-i),diag(nrow=K-i))
  j <- j+K-i
}

# Get coefficients [H]_{k*-1,m} - [H]_{a-1,m}
coeffs <- D %*% post.cov %*% crossprod(X.noninf, solve(lik.cov.noninf))

# Add in column(s) of zeros for likelihood contrast(s) with zero influence
rleinf <- rle(inftf)
tempcoeffs <- matrix(0, nrow=nrow(coeffs), ncol=nrow(contr.ab))
tempcoeffs[,!inftf] <- coeffs
coeffs <- tempcoeffs

# We are only interested in the K-1 (posterior) contrasts that involve k*
kstar <- which(c(0,post.mean)==min(0,post.mean))

# For second place thresholds
k2nd <- which(c(0,post.mean)==sort(c(0,post.mean))[2])

# For CEA treatments only
ceatrt <- c(1, 2, 5, 6, 7, 8, 9, 12, 13, 14, 15, 16, 17, 18, 
            19, 22, 23, 24, 26, 27, 28, 29, 30, 31, 33, 34, 35, 36)
post.mean.subNA <- post.mean
post.mean.subNA[!(1:(K-1) %in% (ceatrt-1))] <- NA
kCEA <- order(c(0,post.mean.subNA), decreasing=FALSE)[1]

# And these contrasts have non-zero elements in the contrast design matrix D in the k* column
contr.kstar <- which(D[,kstar-1] != 0)
contr.k2nd <- which(D[,k2nd-1] != 0)
contr.kCEA <- which(D[,kCEA-1] != 0)[ceatrt[ceatrt!=kCEA] - (ceatrt[ceatrt!=kCEA]>=kCEA)*1]

# So we look in the corresponding rows of the matrix coeffs
coeffs.kstar <- coeffs[contr.kstar,]
coeffs.k2nd <- coeffs[contr.k2nd,]
coeffs.kCEA <- coeffs[contr.kCEA,]

#### BIAS IN ALL DRUG COMPARISONS WITH INACTIVE ####

# Get denominators for each threshold equation by summing the coefficients of
# all DRUG data points. Do this with and without combined regimens.

# Drug treatments
drugtrts <- 9:23
drugtrts.withcombined <- c(9:23, 37:41)

# Which data points compare these to an inactive trt?
drugdats <- which(contr.ab$a %in% 1:3 & contr.ab$b %in% drugtrts)
drugdats.withcombined <- which(contr.ab$a %in% 1:3 & contr.ab$b %in% drugtrts.withcombined)

# Get threshold solutions
threshmat.drugs <- -(D %*% post.mean)[contr.kstar] / rowSums(coeffs.kstar[,drugdats])
threshmat.drugs.withcombined <- -(D %*% post.mean)[contr.kstar] / rowSums(coeffs.kstar[,drugdats.withcombined])

threshmat.k2nd.drugs <- -(D %*% post.mean)[contr.k2nd] / rowSums(coeffs.k2nd[,drugdats])
threshmat.k2nd.drugs.withcombined <- -(D %*% post.mean)[contr.k2nd] / rowSums(coeffs.k2nd[,drugdats.withcombined])

threshmat.ceatrt.drugs <- -(D %*% post.mean)[contr.kCEA] / rowSums(coeffs.kCEA[,drugdats])
threshmat.ceatrt.drugs.withcombined <- -(D %*% post.mean)[contr.kCEA] / rowSums(coeffs.kCEA[,drugdats.withcombined])


# Get positive and negative threshold values, new k*

get.int <- function(x,kstar,trt.sub=1:K){
  if(all(is.infinite(x))){
    hi <- Inf
    lo <- -Inf
    hi.newkstar <- lo.newkstar <- NA
  }
  else if (all(x[!is.infinite(x)]>0)){
    hi <- min(x[!is.infinite(x)])
    hi.newkstar <- trt.sub[which(x==hi) + (which(x==hi) >= which(trt.sub==kstar))*1]
    lo <- -Inf
    lo.newkstar <- NA
  }
  else if (all(x[!is.infinite(x)]<0)){
    hi <- Inf
    hi.newkstar <- NA
    lo <- max(x[!is.infinite(x)])
    lo.newkstar <- trt.sub[which(x==lo) + (which(x==lo) >= which(trt.sub==kstar))*1]
  }
  else {
    hi <- min(x[x>0 & !is.infinite(x)])
    hi.newkstar <- trt.sub[which(x==hi) + (which(x==hi) >= which(trt.sub==kstar))*1]
    lo <- max(x[x<0 & !is.infinite(x)])
    lo.newkstar <- trt.sub[which(x==lo) + (which(x==lo) >= which(trt.sub==kstar))*1]
  }
  return(data.frame(lo=lo,lo.newkstar=lo.newkstar,hi=hi,hi.newkstar=hi.newkstar))
}

thresh.drugs <- get.int(threshmat.drugs,kstar)
thresh.drugs.withcombined <- get.int(threshmat.drugs.withcombined,kstar)

thresh.k2nd.drugs <- get.int(threshmat.k2nd.drugs,k2nd)
thresh.k2nd.drugs.withcombined <- get.int(threshmat.k2nd.drugs.withcombined,k2nd)

thresh.ceatrt.drugs <- get.int(threshmat.ceatrt.drugs, kCEA, ceatrt)
thresh.ceatrt.drugs.withcombined <- get.int(threshmat.ceatrt.drugs.withcombined, kCEA, ceatrt)

## Plot
plotme <- function(thresh, contr.mean, CrI.lo, CrI.hi, rowlabs, xlim, xlab, ylab, ...){
  
  yaxs <- length(contr.mean):1
  
  # split plot in two
  layout(matrix(1:2,nrow=1), widths=c(.2,1))
  
  # plot row labels on left side
  gp <- par(mar=c(5,4,1,0))
  plot(rep(0,length(yaxs)), yaxs, pch=NA, ylim=c(.5,yaxs[1]+.5), ylab="",
       xlab="", yaxt="n", xaxt="n", bty="n")
  text(0, yaxs, labels=rowlabs,xpd=NA)
  title(ylab=ylab, line=2)
  
  # fake plot for setup of right side
  par(mar=c(5,1,1,2))
  plot(contr.mean, yaxs, pch=NA, yaxt="n", xlim=xlim,
       ylim=c(.5,yaxs[1]+.5), ylab="", xlab="",...)
  title(xlab=xlab, line=2)
  
  # reference line
  abline(v=0, lty=2, col="gray")
  
  # combined invariant region
  polygon(rep(c(contr.mean + thresh$lo, rev(contr.mean) + thresh$hi), each=2), 
          c(rep(yaxs,each=2) + c(.5,-.5), rep(rev(yaxs),each=2) + c(-.5,.5)),
          col=rgb(.7,.8,.9,.7),
          border=rgb(.6,.7,.8))
  
  # credible intervals
  segments(y0=yaxs, x0=CrI.lo, x1=CrI.hi, lend=1)
  
  # contrast means
  points(contr.mean, yaxs, pch=21, col="black", bg="white")
  
  # write new optimal treatments at either side
  text(xlim[1], round(length(yaxs)/2), 
       labels=as.expression(substitute(tilde(k)*"* = "*xx,list(xx=thresh$lo.newkstar))), 
       pos=4)
  text(xlim[2], round(length(yaxs)/2),
       labels=as.expression(substitute(tilde(k)*"* = "*xx,list(xx=thresh$hi.newkstar))),
       pos=2)
  
  # write invariant interval below plot
  with(thresh, title(xlab=paste0("Invariant interval about zero: ",lo.newkstar,
                                 " (",formatC(lo,digits=2,format="f"),", ",
                                 formatC(hi,digits=2,format="f"),") ",
                                 hi.newkstar), line=3))
  
  par(gp)
}



pdf("Social Anxiety thresholds - contrast level by group - drugs.pdf", width=8, height=5)
plotme(thresh.drugs, 
       dd.summary$stat[drugdats,"Mean"], 
       dd.summary$quant[drugdats,"2.5%"],
       dd.summary$quant[drugdats,"97.5%"],
       rowlabs=paste(contr.ab[drugdats,"b"],"vs.",contr.ab[drugdats,"a"]),
       xlim=c(-3,2.5), ylab="Drug vs. Inactive Contrasts", xlab="SMD")
dev.off()

pdf("Social Anxiety thresholds - contrast level by group - drugs with combined.pdf", width=8, height=6)
plotme(thresh.drugs.withcombined, 
       dd.summary$stat[drugdats.withcombined,"Mean"], 
       dd.summary$quant[drugdats.withcombined,"2.5%"],
       dd.summary$quant[drugdats.withcombined,"97.5%"],
       rowlabs=paste(contr.ab[drugdats.withcombined,"b"],"vs.",contr.ab[drugdats.withcombined,"a"]),
       xlim=c(-4,1), ylab="Drug vs. Inactive Contrasts", xlab="SMD")
dev.off()

pdf("Social Anxiety thresholds - contrast level by group - drugs - second place.pdf", width=8, height=5)
plotme(thresh.k2nd.drugs, 
       dd.summary$stat[drugdats,"Mean"], 
       dd.summary$quant[drugdats,"2.5%"],
       dd.summary$quant[drugdats,"97.5%"],
       rowlabs=paste(contr.ab[drugdats,"b"],"vs.",contr.ab[drugdats,"a"]),
       xlim=c(-3,2.5), ylab="Drug vs. Inactive Contrasts", xlab="SMD")
dev.off()

pdf("Social Anxiety thresholds - contrast level by group - drugs with combined - second place.pdf", width=8, height=6)
plotme(thresh.k2nd.drugs.withcombined, 
       dd.summary$stat[drugdats.withcombined,"Mean"], 
       dd.summary$quant[drugdats.withcombined,"2.5%"],
       dd.summary$quant[drugdats.withcombined,"97.5%"],
       rowlabs=paste(contr.ab[drugdats.withcombined,"b"],"vs.",contr.ab[drugdats.withcombined,"a"]),
       xlim=c(-4,1), ylab="Drug vs. Inactive Contrasts", xlab="SMD")
dev.off()

pdf("Social Anxiety thresholds - contrast level by group - drugs - CEA treatments.pdf", 
    width=8, height=5)
plotme(thresh.ceatrt.drugs, 
       dd.summary$stat[drugdats,"Mean"], 
       dd.summary$quant[drugdats,"2.5%"],
       dd.summary$quant[drugdats,"97.5%"],
       rowlabs=paste(contr.ab[drugdats,"b"],"vs.",contr.ab[drugdats,"a"]),
       xlim=c(-3.5,2.5), ylab="Drug vs. Inactive Contrasts", xlab="SMD")
dev.off()

pdf("Social Anxiety thresholds - contrast level by group - drugs with combined - CEA treatments.pdf", 
    width=8, height=6)
plotme(thresh.ceatrt.drugs.withcombined, 
       dd.summary$stat[drugdats.withcombined,"Mean"], 
       dd.summary$quant[drugdats.withcombined,"2.5%"],
       dd.summary$quant[drugdats.withcombined,"97.5%"],
       rowlabs=paste(contr.ab[drugdats.withcombined,"b"],"vs.",contr.ab[drugdats.withcombined,"a"]),
       xlim=c(-5,2.5), ylab="Drug vs. Inactive Contrasts", xlab="SMD")
dev.off()

#### BIAS IN ALL PSYCH COMPARISONS WITH INACTIVE ####

# Get denominators for each threshold equation by summing the coefficients of
# all PSYCH data points. Do this with and without combined regimens.

# Psych treatments
psychtrts <- c(4:8, 24:36)
psychtrts.withcombined <- c(4:8, 24:36, 37:41)

# Which data points compare these to an inactive trt?
psychdats <- which(contr.ab$a %in% 1:3 & contr.ab$b %in% psychtrts)
psychdats.withcombined <- which(contr.ab$a %in% 1:3 & contr.ab$b %in% psychtrts.withcombined)

# Get threshold solutions
threshmat.psych <- -(D %*% post.mean)[contr.kstar] / rowSums(coeffs.kstar[,psychdats])
threshmat.psych.withcombined <- -(D %*% post.mean)[contr.kstar] / rowSums(coeffs.kstar[,psychdats.withcombined])

threshmat.k2nd.psych <- -(D %*% post.mean)[contr.k2nd] / rowSums(coeffs.k2nd[,psychdats])
threshmat.k2nd.psych.withcombined <- -(D %*% post.mean)[contr.k2nd] / rowSums(coeffs.k2nd[,psychdats.withcombined])

threshmat.ceatrt.psych <- -(D %*% post.mean)[contr.kCEA] / rowSums(coeffs.kCEA[,psychdats])
threshmat.ceatrt.psych.withcombined <- -(D %*% post.mean)[contr.kCEA] / rowSums(coeffs.kCEA[,psychdats.withcombined])


# Get positive and negative threshold values, new k*
thresh.psych <- get.int(threshmat.psych,kstar)
thresh.psych.withcombined <- get.int(threshmat.psych.withcombined,kstar)

thresh.k2nd.psych <- get.int(threshmat.k2nd.psych,k2nd)
thresh.k2nd.psych.withcombined <- get.int(threshmat.k2nd.psych.withcombined,k2nd)

thresh.ceatrt.psych <- get.int(threshmat.ceatrt.psych, kCEA, ceatrt)
thresh.ceatrt.psych.withcombined <- get.int(threshmat.ceatrt.psych.withcombined, kCEA, ceatrt)

## Plot
pdf("Social Anxiety thresholds - contrast level by group - psych.pdf", width=8, height=5.5)
plotme(thresh.psych, 
       dd.summary$stat[psychdats,"Mean"], 
       dd.summary$quant[psychdats,"2.5%"],
       dd.summary$quant[psychdats,"97.5%"],
       rowlabs=paste(contr.ab[psychdats,"b"],"vs.",contr.ab[psychdats,"a"]),
       xlim=c(-3,2), ylab="Psych vs. Inactive Contrasts", xlab="SMD")
dev.off()

pdf("Social Anxiety thresholds - contrast level by group - psych with combined.pdf", width=8, height=6)
plotme(thresh.psych.withcombined, 
       dd.summary$stat[psychdats.withcombined,"Mean"], 
       dd.summary$quant[psychdats.withcombined,"2.5%"],
       dd.summary$quant[psychdats.withcombined,"97.5%"],
       rowlabs=paste(contr.ab[psychdats.withcombined,"b"],"vs.",contr.ab[psychdats.withcombined,"a"]),
       xlim=c(-5,2), ylab="Psych vs. Inactive Contrasts", xlab="SMD")
dev.off()

pdf("Social Anxiety thresholds - contrast level by group - psych - second place.pdf", width=8, height=5.5)## Plot
plotme(thresh.k2nd.psych, 
       dd.summary$stat[psychdats,"Mean"], 
       dd.summary$quant[psychdats,"2.5%"],
       dd.summary$quant[psychdats,"97.5%"],
       rowlabs=paste(contr.ab[psychdats,"b"],"vs.",contr.ab[psychdats,"a"]),
       xlim=c(-3,1), ylab="Psych vs. Inactive Contrasts", xlab="SMD")
dev.off()

pdf("Social Anxiety thresholds - contrast level by group - psych with combined - second place.pdf", width=8, height=6)
plotme(thresh.k2nd.psych.withcombined, 
       dd.summary$stat[psychdats.withcombined,"Mean"], 
       dd.summary$quant[psychdats.withcombined,"2.5%"],
       dd.summary$quant[psychdats.withcombined,"97.5%"],
       rowlabs=paste(contr.ab[psychdats.withcombined,"b"],"vs.",contr.ab[psychdats.withcombined,"a"]),
       xlim=c(-5,1), ylab="Psych vs. Inactive Contrasts", xlab="SMD")
dev.off()

pdf("Social Anxiety thresholds - contrast level by group - psych - CEA treatments.pdf", 
    width=8, height=5.5)## Plot
plotme(thresh.ceatrt.psych, 
       dd.summary$stat[psychdats,"Mean"], 
       dd.summary$quant[psychdats,"2.5%"],
       dd.summary$quant[psychdats,"97.5%"],
       rowlabs=paste(contr.ab[psychdats,"b"],"vs.",contr.ab[psychdats,"a"]),
       xlim=c(-5,1.5), ylab="Psych vs. Inactive Contrasts", xlab="SMD")
dev.off()

pdf("Social Anxiety thresholds - contrast level by group - psych with combined - CEA treatments.pdf", 
    width=8, height=6)
plotme(thresh.ceatrt.psych.withcombined, 
       dd.summary$stat[psychdats.withcombined,"Mean"], 
       dd.summary$quant[psychdats.withcombined,"2.5%"],
       dd.summary$quant[psychdats.withcombined,"97.5%"],
       rowlabs=paste(contr.ab[psychdats.withcombined,"b"],"vs.",contr.ab[psychdats.withcombined,"a"]),
       xlim=c(-5,1.5), ylab="Psych vs. Inactive Contrasts", xlab="SMD")
dev.off()

#### BIAS IN DRUG AND PSYCH SIMULTANEOUSLY ####

# Excluding combined treatments
# Show PSYCH on y-axis and DRUG on x-axis, so divide intercept and DRUG coeffs by PSYCH coeffs

linedat <- data.frame(intercept= -(D %*% post.mean)[contr.kstar] / rowSums(coeffs.kstar[,psychdats]),
                      gradient= -rowSums(coeffs.kstar[,drugdats]) / rowSums(coeffs.kstar[,psychdats]))

# Output to PDF
pdf("Social Anxiety thresholds - contrast level by group - 2D.pdf", width=10, height=8)

# Set up plot
plot(NA, xlim=c(-2,3), ylim=c(-2,2), xlab="SMD: Drug vs. Inactive", ylab="SMD: Psych vs. Inactive")

# Axes
abline(h=0, lty=2, col="gray")
abline(v=0, lty=2, col="gray")

# Function to get intercept values of two lines
get.intercept <- function(line1, line2){
  if(all(line1==line2)) return(c(NA,NA))
  
  x <- (line2$intercept - line1$intercept) / (line1$gradient - line2$gradient)
  y <- line1$gradient * x + line1$intercept
  return(c(x,y))
}

# Draw invariant region
polygon(rbind(get.intercept(linedat[36,], linedat[39,]),
              get.intercept(linedat[36,], linedat[2,]),
              get.intercept(linedat[3,], linedat[2,]),
              get.intercept(linedat[3,], linedat[39,])), 
        col=rgb(.7,.8,.9,.7), border=NA)

# Lines
apply(linedat, 1, function(x) abline(coef=x))

# Write new optimal treatments at thresholds
text(0.20, 1.65, labels=expression(tilde(k)*"* = 3"))
text(2.25, 0.48, labels=expression(tilde(k)*"* = 2"))
text(0.49, -0.75, labels=expression(tilde(k)*"* = 36"))
polygon(c(-1.5, -1.5, -1.15, -1.15), c(0.2, 0.4, 0.4, 0.2), col=rgb(1,1,1,.8), border=NA)
text(-1.32, 0.28, labels=expression(tilde(k)*"* = 39"))

dev.off()

## Second place ##
# Excluding combined treatments
# Show PSYCH on y-axis and DRUG on x-axis, so divide intercept and DRUG coeffs by PSYCH coeffs

linedat.k2nd <- data.frame(intercept= -(D %*% post.mean)[contr.k2nd] / rowSums(coeffs.k2nd[,psychdats]),
                           gradient= -rowSums(coeffs.k2nd[,drugdats]) / rowSums(coeffs.k2nd[,psychdats]))

# Output to PDF
pdf("Social Anxiety thresholds - contrast level by group - 2D second place.pdf", width=10, height=8)

# Set up plot
plot(NA, xlim=c(-2,3), ylim=c(-2,2), xlab="SMD: Drug vs. Inactive", ylab="SMD: Psych vs. Inactive")

# Axes
abline(h=0, lty=2, col="gray")
abline(v=0, lty=2, col="gray")

# Draw invariant region
polygon(rbind(get.intercept(linedat.k2nd[38,], linedat.k2nd[40,]),
              get.intercept(linedat.k2nd[38,], linedat.k2nd[3,]),
              get.intercept(linedat.k2nd[2,], linedat.k2nd[3,]),
              get.intercept(linedat.k2nd[2,], linedat.k2nd[40,])), 
        col=rgb(.7,.8,.9,.7), border=NA)

# Lines
apply(linedat.k2nd, 1, function(x) abline(coef=x))

# Write new optimal treatments at thresholds
text(0.85, 1.1, labels=expression(tilde(k)*"* = 3"))
text(2.1, 0.4, labels=expression(tilde(k)*"* = 2"))
text(0.49, -0.75, labels=expression(tilde(k)*"* = 41"))
polygon(c(-0.9, -0.9, -0.6, -0.6), c(0.15, 0.4, 0.4, 0.15), col=rgb(1,1,1,.8), border=NA)
text(-0.7, 0.3, labels=expression(tilde(k)*"* = 39"))

dev.off()

## CEA treatments only ##
# Excluding combined treatments
# Show PSYCH on y-axis and DRUG on x-axis, so divide intercept and DRUG coeffs by PSYCH coeffs

linedat.ceatrt <- data.frame(intercept= -(D %*% post.mean)[contr.kCEA] / rowSums(coeffs.kCEA[,psychdats]),
                             gradient= -rowSums(coeffs.kCEA[,drugdats]) / rowSums(coeffs.kCEA[,psychdats]))

# Output to PDF
pdf("Social Anxiety thresholds - contrast level by group - 2D CEA treatments.pdf", width=10, height=8)

# Set up plot
plot(NA, xlim=c(-10,10), ylim=c(-20,5), xlab="SMD: Drug vs. Inactive", ylab="SMD: Psych vs. Inactive")

# Axes
abline(h=0, lty=2, col="gray")
abline(v=0, lty=2, col="gray")

# Draw invariant region
polygon(rbind(get.intercept(linedat.ceatrt[17,], linedat.ceatrt[8,]),
              get.intercept(linedat.ceatrt[8,], linedat.ceatrt[2,]),
              get.intercept(linedat.ceatrt[2,], linedat.ceatrt[26,]),
              get.intercept(linedat.ceatrt[26,], linedat.ceatrt[27,]),
              get.intercept(linedat.ceatrt[27,], linedat.ceatrt[10,]),
              get.intercept(linedat.ceatrt[10,], linedat.ceatrt[14,]),
              get.intercept(linedat.ceatrt[14,], linedat.ceatrt[17,])
              ), 
        col=rgb(.7,.8,.9,.7), border=NA)

# Lines
apply(linedat.ceatrt, 1, function(x) abline(coef=x))

# Write new optimal treatments at thresholds

dev.off()


### Try to get invariant polygon automatically... ###

# Array of intercepts of threshold lines (K-1)x(K-1)x{x,y}
intarray <- array(dim=c(K-1,K-1,2))
for (i in 1:(K-1)){
  for (j in 1:(K-1)){
    if (i != j) intarray[i,j,] <- get.intercept(linedat[i,], linedat[j,])
  }
}

# Get basic edges using intercepts with axes
axesint <- data.frame(x=-linedat$intercept/linedat$gradient,
                      y=linedat$intercept)
polyedges <- c()