############################################################
# CG159 Social Anxiety: Contrast Level Threshold Analysis
#   - Performed on standardised mean difference scale (SMD)
############################################################

source("../../../Functions/recon_vcov.R")
source("../../../Functions/nma_thresh.R")
source("../../../Functions/thresh_forest.R")
source("../../../Functions/d_ab2i.R")

#### STEP 0 ####
# Read CODA data
source("./Social Anxiety - read CODA.R")

#### STEP 1 ####
# First, we must reconstruct the likelihood covariance matrix.

# Create contrast design matrix X. We'll use the treatment details from the study 
# level data for convenience.
trt.dat <- read.table(file = "../WinBUGS/study data.txt", header = TRUE, sep = "\t")[,1:6]
K <- with(trt.dat, length(unique(c(t.1, t.2, t.3, t.4, t.5)))) -1  # Number of treatments , -1 for NA

# work out which contrasts have data
contr.ab <- data.frame(a=c(), b=c())

for(i in 1:nrow(trt.dat)){
   rowi <- trt.dat[i,which(!is.na(trt.dat[i,2:6]))+1] # non NA elements of ith row
   
   # get contrast from all combinations of treatments
   trtcomb <- combn(rowi, 2, function(x) sapply(x, as.numeric))
   
   a <- apply(trtcomb, 2, min)
   b <- apply(trtcomb, 2, max)
   
   # remove contrasts of treatments against themselves
   iseq <- a==b
   a <- a[!iseq]
   b <- b[!iseq]
   
   if (!all(iseq)) contr.ab <- rbind(contr.ab, cbind(a,b))
}

contr.ab <- unique(contr.ab[order(contr.ab$a, contr.ab$b),])


# Contrast design matrix
X <- matrix(0, nrow=nrow(contr.ab), ncol=K-1)
for (i in 1:nrow(X)){
  if (contr.ab[i,"a"] > 1) X[i, contr.ab[i,"a"] - 1]  <- -1
  if (contr.ab[i,"b"] > 1)   X[i, contr.ab[i,"b"] - 1]    <- 1
}


# Reconstruct using NNLS
# Class model is used, so use the prior mean precision from the gamma distribution
prior.prec <- rep(3.9/0.35, 40)

# Other than for treatment 3
prior.prec[2] <- 0.0001

lik.cov <- recon_vcov(post.cov, prior.vcov=diag(1/prior.prec), X=X)

# Some variances are estimated to be infinite, so remove these contrasts for thresholding
inftf <- is.infinite(diag(lik.cov))
X.noninf <- X[!inftf,]
lik.cov.noninf <- diag(diag(lik.cov)[!inftf])

# Get indices of contrasts in likelihood in full contrast vector
d.a <- d.b <- vector(length=nrow(X))
for (i in 1:nrow(X)){
  d.a[i] <- ifelse(any(X[i,]==-1),which(X[i,]==-1),0)+1
  d.b[i] <- ifelse(any(X[i,]==1),which(X[i,]==1),0)+1
}

d.i <- d_ab2i(d.a, d.b, K=K)

#### STEP 2 ####
# Now we can perform thresholding at the contrast level.

# Thresholds are
thresh <- nma_thresh(X.noninf, post.mean, lik.cov.noninf, post.cov, opt.max=FALSE)

# Merge removed contrasts back in
thresh$d.i <- d.i[!inftf]
thresh <- merge(data.frame(d.i=d.i), thresh, by="d.i", all.x=TRUE)

thresh[inftf, c("lo","hi")] <- c(-Inf, Inf)

# Label the contrasts and display using a forest plot, along with 95% posterior
# credible intervals as reported in CG150.1

#contr <- paste0("d[",d.a,d.b,"]")
contr <- paste0(d.b," vs. ",d.a)

contr.mean <- dd.mean[d.i]
CI2.5 <- dd.summary$quantiles[d.i,"2.5%"]
CI97.5 <- dd.summary$quantiles[d.i,"97.5%"]
print(thresh <- as.data.frame(cbind(contr,contr.mean,CI2.5,CI97.5,thresh)), row.names=FALSE)

## COLOUR
pdf("Social Anxiety thresholds - contrast level.pdf", width=13, height=16)
with(thresh,
     thresh_forest(means=contr.mean, t.lo=lo, t.hi=hi, kstar.lo=lo.newkstar, kstar.hi=hi.newkstar,
                   CI.lo=CI2.5, CI.hi=CI97.5, label=contr, label.title="Contrast",
                   xlab="Standardised Mean Difference", xlim=c(-4,3), 
                   refline=0, digits=2)
)
dev.off()

## B&W
pdf("Social Anxiety thresholds - contrast level BW.pdf", width=13, height=16)
with(thresh,
     thresh_forest(means=contr.mean, t.lo=lo, t.hi=hi, kstar.lo=lo.newkstar, kstar.hi=hi.newkstar,
                   CI.lo=CI2.5, CI.hi=CI97.5, label=contr, label.title="Contrast",
                   xlab="Standardised Mean Difference", xlim=c(-4,3), 
                   refline=0, digits=2, greyscale=TRUE)
)
dev.off()

## Only report contrasts where the threshold is less than 3 SMD
cutoff <- 3
thresh.cut <- thresh[thresh$lo > -cutoff | thresh$hi < cutoff, ]

## COLOUR
pdf("Social Anxiety thresholds - contrast level CUTOFF.pdf", width=13, height=10)
with(thresh.cut,
     thresh_forest(means=contr.mean, t.lo=lo, t.hi=hi, kstar.lo=lo.newkstar, kstar.hi=hi.newkstar,
                   CI.lo=CI2.5, CI.hi=CI97.5, label=contr, label.title="Contrast",
                   xlab="Standardised Mean Difference", xlim=c(-4,3), 
                   refline=0, digits=2)
)
dev.off()

## Sorted with smallest thresholds first
absmin <- function(x) min(abs(x))
thresh.cut.sort <- thresh.cut[order(apply(thresh.cut[,c("lo","hi")], 1, absmin)),]

## COLOUR
pdf("Social Anxiety thresholds - contrast level CUTOFF SORT.pdf", width=13, height=10)
with(thresh.cut.sort,
     thresh_forest(means=contr.mean, t.lo=lo, t.hi=hi, kstar.lo=lo.newkstar, kstar.hi=hi.newkstar,
                   CI.lo=CI2.5, CI.hi=CI97.5, label=contr, label.title="Contrast",
                   xlab="Standardised Mean Difference", xlim=c(-4,3), 
                   refline=0, digits=2)
)
dev.off()



## With correction for residual values in posterior covariance matrix. Round
## these values to zero below a certain cutoff (0.005), based on the MC error of
## contrasts.
# 
# post.cov.cut <- post.cov
# post.cov.cut[which(abs(post.cov)<0.005)] <- 0
# 
# thresh.cut <- nma_thresh(X.noninf, post.mean, lik.cov.noninf, post.cov.cut, twoway=TRUE, opt.max=FALSE)
# 
# # Merge removed contrasts back in
# thresh.cut$d.i <- d.i[!inftf]
# thresh.cut <- merge(data.frame(d.i=d.i), thresh.cut, by="d.i", all.x=TRUE)
# 
# print(thresh.cut <- as.data.frame(cbind(contr,contr.mean,CI2.5,CI97.5,thresh.cut)), row.names=FALSE)
# 
# ## COLOUR
# pdf("Social Anxiety thresholds - contrast level CUTOFF.pdf", width=13, height=16)
# with(thresh.cut,
#      thresh_forest(means=contr.mean, t.lo=lo, t.hi=hi, kstar.lo=lo.newkstar, kstar.hi=hi.newkstar,
#                    CI.lo=CI2.5, CI.hi=CI97.5, label=contr, label.title="Contrast",
#                    xlab="Standardised Mean Difference", xlim=c(-2,1), 
#                    refline=0, sigfig=3)
# )
# dev.off()
# 
# ## B&W
# pdf("Social Anxiety thresholds - contrast level CUTOFF BW.pdf", width=13, height=16)
# with(thresh.cut,
#      thresh_forest(means=contr.mean, t.lo=lo, t.hi=hi, kstar.lo=lo.newkstar, kstar.hi=hi.newkstar,
#                    CI.lo=CI2.5, CI.hi=CI97.5, label=contr, label.title="Contrast",
#                    xlab="Standardised Mean Difference", xlim=c(-2,1), 
#                    refline=0, sigfig=3, greyscale=TRUE)
# )
# dev.off()

##### Thresholds for second place #####

# Get thresholds
#thresh.k2nd <- as.data.frame(do.call(rbind, apply(threshmat.k2nd, 2, get.int, kstar=k2nd)))
thresh.k2nd <- nma_thresh(X.noninf, post.mean, lik.cov.noninf, post.cov, opt.max=FALSE, trt.rank=2)

# Merge removed contrasts back in
thresh.k2nd$d.i <- d.i[!inftf]
thresh.k2nd <- merge(data.frame(d.i=d.i), thresh.k2nd, by="d.i", all.x=TRUE)

thresh.k2nd[inftf, c("lo","hi")] <- c(-Inf, Inf)

print(thresh.k2nd <- as.data.frame(cbind(contr,contr.mean,CI2.5,CI97.5,thresh.k2nd)), row.names=FALSE)

## COLOUR
pdf("Social Anxiety thresholds - contrast level - second place.pdf", width=13, height=16)
with(thresh.k2nd,
     thresh_forest(means=contr.mean, t.lo=lo, t.hi=hi, kstar.lo=lo.newkstar, kstar.hi=hi.newkstar,
                   CI.lo=CI2.5, CI.hi=CI97.5, label=contr, label.title="Contrast",
                   xlab="Standardised Mean Difference", xlim=c(-4,3), 
                   refline=0, digits=2)
)
dev.off()

thresh.k2nd.cut <- thresh.k2nd[thresh.k2nd$lo > -cutoff | thresh.k2nd$hi < cutoff, ]

## COLOUR
pdf("Social Anxiety thresholds - contrast level - second place CUTOFF.pdf", width=13, height=10)
with(thresh.k2nd.cut,
     thresh_forest(means=contr.mean, t.lo=lo, t.hi=hi, kstar.lo=lo.newkstar, kstar.hi=hi.newkstar,
                   CI.lo=CI2.5, CI.hi=CI97.5, label=contr, label.title="Contrast",
                   xlab="Standardised Mean Difference", xlim=c(-4,3), 
                   refline=0, digits=2)
)
dev.off()

thresh.k2nd.cut.sort <- thresh.k2nd.cut[order(apply(thresh.k2nd.cut[,c("lo","hi")], 1, absmin)),]

## COLOUR
pdf("Social Anxiety thresholds - contrast level - second place CUTOFF SORT.pdf", width=13, height=10)
with(thresh.k2nd.cut.sort,
     thresh_forest(means=contr.mean, t.lo=lo, t.hi=hi, kstar.lo=lo.newkstar, kstar.hi=hi.newkstar,
                   CI.lo=CI2.5, CI.hi=CI97.5, label=contr, label.title="Contrast",
                   xlab="Standardised Mean Difference", xlim=c(-4,3), 
                   refline=0, digits=2)
)
dev.off()

##### Thresholds for CEA treatments only #####

# Get thresholds
ceatrt <- c(1, 2, 5, 6, 7, 8, 9, 12, 13, 14, 15, 16, 17, 18, 
            19, 22, 23, 24, 26, 27, 28, 29, 30, 31, 33, 34, 35, 36)
thresh.ceatrt <- nma_thresh(X.noninf, post.mean, lik.cov.noninf, post.cov, opt.max=FALSE,
                            trt.sub=ceatrt)

# Merge removed contrasts back in
thresh.ceatrt$d.i <- d.i[!inftf]
thresh.ceatrt <- merge(data.frame(d.i=d.i), thresh.ceatrt, by="d.i", all.x=TRUE)

thresh.ceatrt[inftf, c("lo","hi")] <- c(-Inf, Inf)

print(thresh.ceatrt <- as.data.frame(cbind(contr,contr.mean,CI2.5,CI97.5,thresh.ceatrt)), row.names=FALSE)

## COLOUR
pdf("Social Anxiety thresholds - contrast level - CEA treatments.pdf", width=13, height=16)
with(thresh.ceatrt,
     thresh_forest(means=contr.mean, t.lo=lo, t.hi=hi, kstar.lo=lo.newkstar, kstar.hi=hi.newkstar,
                   CI.lo=CI2.5, CI.hi=CI97.5, label=contr, label.title="Contrast",
                   xlab="Standardised Mean Difference", xlim=c(-4,3), 
                   refline=0, digits=2)
)
dev.off()

thresh.ceatrt.cut <- thresh.ceatrt[thresh.ceatrt$lo > -cutoff | thresh.ceatrt$hi < cutoff, ]

## COLOUR
pdf("Social Anxiety thresholds - contrast level - CEA treatments CUTOFF.pdf", width=13, height=10)
with(thresh.ceatrt.cut,
     thresh_forest(means=contr.mean, t.lo=lo, t.hi=hi, kstar.lo=lo.newkstar, kstar.hi=hi.newkstar,
                   CI.lo=CI2.5, CI.hi=CI97.5, label=contr, label.title="Contrast",
                   xlab="Standardised Mean Difference", xlim=c(-4,3), 
                   refline=0, digits=2)
)
dev.off()

thresh.ceatrt.cut.sort <- thresh.ceatrt.cut[order(apply(thresh.ceatrt.cut[,c("lo","hi")], 1, absmin)),]

## COLOUR
pdf("Social Anxiety thresholds - contrast level - CEA treatments CUTOFF SORT.pdf", width=13, height=10)
with(thresh.ceatrt.cut.sort,
     thresh_forest(means=contr.mean, t.lo=lo, t.hi=hi, kstar.lo=lo.newkstar, kstar.hi=hi.newkstar,
                   CI.lo=CI2.5, CI.hi=CI97.5, label=contr, label.title="Contrast",
                   xlab="Standardised Mean Difference", xlim=c(-4,3), 
                   refline=0, digits=2)
)
dev.off()
