############################################################
# CG159 Social Anxiety: Study Level CEA Thresholds
############################################################

source("../../../Functions/thresh_forest.R")

##### STEP 1 #####
# Read CODA data to get posterior info
source("./Social Anxiety - read CODA.R")

#### STEP 2 ####
## Net benefit parameters

# WTP
lambda <- 20000

# Utilities (expected value)
U.socanx <- 0.659
U.well <- 0.866

# Costs (expected value)
C.NHS.socanx <- 1037
C.NHS.well <- 645

costs <- read.csv("../costs.csv")
readorder <- costs$k # grab the ordering used in the CEA for easier comparison later

costs <- costs[order(costs$k),]

# Which treatments actually went in to CEA?
ceatrt <- costs$k

## Expected Conditional Net Benefits
attach(costs)

NB.recover <- prob.relapse * ((U.socanx + 0.5*(U.well - U.socanx))*lambda - 
                                (C.trt26 + C.GP26 + 0.5*(C.NHS.socanx + C.NHS.well))) +
  (1-prob.relapse) * (U.well * lambda - (C.trt26 + C.GP26 + C.NHS.well)) -
  (C.trt12 + C.GP12)

NB.norecover <- U.socanx * lambda - C.NHS.socanx - C.trt12 - C.GP12

detach(costs)

# Beta parameter for LOR = - beta * pi/sqrt(3) * SMD
beta.mean <- 1.154
beta.sd <- 0.1008

# Mean effect parameter for deriving P(response)
meanA <- -2.629
sdA <- sqrt(1/0.81)

##### STEP 3 #####
## Derive net benefits

# Find mean of logitnormal distribution, using code from logitnorm::momentsLogitnorm
# but leaving out the second calculation for the variance (we don't need that)
meanLogitnorm <- function(mu, sigma, abs.tol = 0, subdivisions = 100L, ...)
{
  fExp <- function(x) plogis(x) * dnorm(x, mean = mu, sd = sigma)
  integrate(fExp, -Inf, Inf, abs.tol = abs.tol, subdivisions = subdivisions,...)$value
}

# SD of product of two independent RVs
sdprod <- function(sdx, Ex, sdy, Ey){
  sqrt(sdx^2 * sdy^2  + sdx^2 * Ey^2  + sdy^2 * Ex^2)
}

## Expected Net Benefit calculation

cal.mean <- - pi/sqrt(3) * beta.mean
cal.sd <- beta.sd * pi/sqrt(3)

# Compare LOR of response (gCal) with WinBUGS output
gCal <- data.frame(mean= cal.mean * post.mean[ceatrt[-1]-1],
                   sd= sdprod(cal.sd, cal.mean,
                              post.summary$stat[ceatrt[-1]-1,"SD"],
                              post.mean[ceatrt[-1]-1]))
cbind(ceatrt[-1], gCal)

# Compare recovery probabilities (T) with WinBUGS output
Tprob <- mapply(logitnorm::momentsLogitnorm,
                mu=meanA + c(0,gCal$mean), sigma=sqrt(sdA^2 + c(0,gCal$sd)^2))
cbind(ceatrt, Tprob["mean",], sqrt(Tprob["var",]))

.evalNB <- function(badj=0, infl=0, mean.d, sd.d, NB.norec, NB.rec){
  
  prob.rec <- meanLogitnorm(meanA + cal.mean * (mean.d +badj*infl), 
                            sqrt(sdA^2 + 
                                   sdprod(cal.sd, cal.mean, sd.d, (mean.d +badj*infl))^2))
  
  prob.rec * NB.rec + (1-prob.rec) * NB.norec
}

evalNB <- function(badj=0, infl=0, mean.d, sd.d, NB.norec, NB.rec){
  sapply(badj,
         function(x) mapply(.evalNB,
                            mean.d=mean.d, sd.d=sd.d,
                            NB.norec=NB.norec, NB.rec=NB.rec,
                            infl=infl,
                            MoreArgs=list(badj=x)))
}

NB0 <- evalNB(mean.d=c(0,post.mean)[ceatrt], sd.d=c(0,post.summary$stat[,"SD"])[ceatrt],
              NB.norec=NB.norecover, NB.rec=NB.recover)

# Optimal treatment k* using Net Benefit is
kstar <- order(NB0,decreasing=TRUE)[1]

########################################################################################
## Below here are calculations specific to the contrast- and study-level analyses

##### STUDY LEVEL ANALYSIS #####
## Get influence matrix
# Read study data
dat.rawl <- read.table(file = "../WinBUGS/study data.txt", header = TRUE, sep = "\t")

n <- nrow(dat.rawl)   # Number of studies

# Turn wide study data into long with one row for each arm
dat.raw <- reshape(dat.rawl, varying=c("t.2","y.2","Var.2","t.3","y.3","Var.3",
                                       "t.4","y.4","Var.4","t.5","y.5","Var.5"), 
                   timevar="arm", idvar="studyID", direction="long")

# Sort data by study and contrast, removing NA rows
dat.raw <- dat.raw[order(dat.raw$studyID, dat.raw$arm, dat.raw$y, na.last=NA),]

K <- length(unique(c(dat.raw$t.1, dat.raw$t)))    # Number of treatments
N <- nrow(dat.raw)    # Number of data points

# Add in 95% CIs
dat.raw$CI2.5 <- with(dat.raw, y + qnorm(0.025)*sqrt(Var))
dat.raw$CI97.5 <- with(dat.raw, y + qnorm(0.975)*sqrt(Var))

# Study labels
dat.raw$lab <- with(dat.raw, paste0(studyID," (",t," vs ",t.1,")"))

# Create likelihood covariance matrix
require(Matrix)
V.diag <- as.list(rep(NA,n))
attach(dat.rawl)
for (i in 1:n){
  if (n.arms[i] == 2){
    V.diag[[i]] <- Var.2[i]
  }
  else {
    V.diag[[i]] <- matrix(V[i], nrow=n.arms[i]-1, ncol=n.arms[i]-1)
    tempVar <- c(Var.2[i], Var.3[i], Var.4[i], Var.5[i])
    diag(V.diag[[i]]) <- tempVar[!is.na(tempVar)]
  }
}
detach(dat.rawl)
V <- bdiag(V.diag)

# Posterior covariance submatrix is
Bstar <- post.cov.ddelta[1:(K-1),K:(K-1+N)]

# Influence matrix is
inflmat <- Bstar %*% solve(V)

## Plot NBs for a range of bias adjustments for a data point
id <- 18 # Pick a data point
badj <- -200:200/10
NBplot <- evalNB(badj=badj, infl=c(0,inflmat[ceatrt[-1]-1,id]),
                 mean.d=c(0,post.mean)[ceatrt], sd.d=c(0,post.summary$stat[,"SD"])[ceatrt],
                 NB.norec=NB.norecover, NB.rec=NB.recover)

matplot(badj, t(NBplot), type="l", 
        ylab="Net Benefit", xlab=paste("Bias Adjustment to data point ",id))

## Create threshold matrix by finding zeros of the NB contrasts involving k*
require(rootSolve)

# Evaluate net benefit contrasts
# Add dummy parameter extendInt to allow this to be passed via uniroot.all to uniroot.
evalNBcontr <- function(badj, row, col, extendInt=NULL){
  trt <- row + (row>=kstar)*1
  
  return(
    evalNB(badj=badj, infl=rbind(0,inflmat[ceatrt[-1]-1,])[row,col],
           mean.d=c(0,post.mean)[ceatrt[trt]], 
           sd.d=c(0,post.summary$stat[,"SD"])[ceatrt[trt]],
           NB.norec=NB.norecover[trt], NB.rec=NB.recover[trt]) -
      evalNB(badj=badj, infl=rbind(0,inflmat[ceatrt[-1]-1,])[row,col],
             mean.d=c(0,post.mean)[ceatrt[kstar]], 
             sd.d=c(0,post.summary$stat[,"SD"])[ceatrt[kstar]],
             NB.norec=NB.norecover[kstar], NB.rec=NB.recover[kstar])
  )
}


absmin <- function(x) ifelse(identical(x,numeric(0)), NA, x[which.min(abs(x))])

# Threshold array, one matrix for negative thresholds and one for positive
threshmat <- array(NA, dim=c(length(ceatrt)-1, N, 2))
for(i in 1:(length(ceatrt)-1)){ # row i for treatment
  for(j in 1:N){ # col j for data point
    rts <- uniroot.all(evalNBcontr, interval=c(-20,20), row=i, col=j, 
                       extendInt="yes")
    
    if(any(rts<0)) threshmat[i,j,1] <- max(rts[rts<0])
    if(any(rts>0)) threshmat[i,j,2] <- min(rts[rts>0])
  }
}

# Plot NB contrast against bias adjustment for a given data point
matplot(badj, t(evalNBcontr(badj, row=1:(length(ceatrt)-1), col=id)), type="l",
        ylab="NB contrast against k*", xlab=paste("Bias Adjustment to data point ",id))
abline(h=0, v=0)
points(x=c(threshmat[,id,]), y=rep(0,(length(ceatrt)-1)*2))

# Return the bias invariant interval for each data point
get.int <- function(x, trt.code){
  if (all(is.na(x))){ 
    hi <- Inf
    lo <- -Inf
    hi.newkstar <- lo.newkstar <- NA
  }
  else if (all(x[!is.na(x)]>0)){
    hi <- min(x[!is.na(x)])
    hi.newkstar <- trt.code[which(x[,2]==hi) + (which(x[,2]==hi) >= kstar)*1]
    lo <- -Inf
    lo.newkstar <- NA
  }
  else if (all(x[!is.na(x)]<0)){
    hi <- Inf
    hi.newkstar <- NA
    lo <- max(x[!is.na(x)])
    lo.newkstar <- trt.code[which(x[,1]==lo) + (which(x[,1]==lo) >= kstar)*1]
  }
  else {
    hi <- min(x[x>0 & !is.na(x)])
    hi.newkstar <- trt.code[which(x[,2]==hi) + (which(x[,2]==hi) >= kstar)*1]
    lo <- max(x[x<0 & !is.na(x)])
    lo.newkstar <- trt.code[which(x[,1]==lo) + (which(x[,1]==lo) >= kstar)*1]
  }
  return(data.frame(lo=lo,lo.newkstar=lo.newkstar,hi=hi,hi.newkstar=hi.newkstar))
}

thresh <- as.data.frame(do.call(rbind, apply(threshmat, 2, get.int, trt.code=ceatrt)))

# Merge with study details
thresh <- cbind(dat.raw, thresh)

## COLOUR
pdf("Social Anxiety thresholds - study level - net benefit.pdf", width=13, height=30)
with(thresh,
     thresh_forest(means=y, t.lo=lo, t.hi=hi, kstar.lo=lo.newkstar, kstar.hi=hi.newkstar,
                   CI.lo=CI2.5, CI.hi=CI97.5, label=lab, label.title="Study (Contrast)",
                   xlab="Standardised Mean Difference", xlim=c(-4,3), 
                   mean.title="SMD", CI.title="95% Confidence Interval",
                   refline=0, digits=2)
)
dev.off()

# Show only contrasts with thresholds less than a cutoff
cutoff <- 10
thresh.cut <- thresh[thresh$lo > -cutoff | thresh$hi < cutoff,]

pdf("Social Anxiety thresholds - study level - net benefit CUT.pdf", width=13, height=10)
with(thresh.cut,
     thresh_forest(means=y, t.lo=lo, t.hi=hi, kstar.lo=lo.newkstar, kstar.hi=hi.newkstar,
                   CI.lo=CI2.5, CI.hi=CI97.5, label=lab, label.title="Study (Contrast)",
                   xlab="Standardised Mean Difference", xlim=c(-4,3), 
                   mean.title="SMD", CI.title="95% Confidence Interval",
                   refline=0, digits=2)
)
dev.off()
