############################################################
# CG159 Social Anxiety: Study Level Threshold Analysis
# for "David Clark" effect
############################################################

#### STEP 0 ####
# Read CODA data
source("./Social Anxiety - read CODA.R")

# Read study data
dat.rawl <- read.table(file = "../WinBUGS/study data.txt", header = TRUE, sep = "\t")

n <- nrow(dat.rawl)   # Number of studies

# Turn wide study data into long with one row for each arm
dat.raw <- reshape(dat.rawl, varying=c("t.2","y.2","Var.2","t.3","y.3","Var.3",
                                       "t.4","y.4","Var.4","t.5","y.5","Var.5"), 
                   timevar="arm", idvar="studyID", direction="long")

# Sort data by study and contrast, removing NA rows
dat.raw <- dat.raw[order(dat.raw$studyID, dat.raw$arm, dat.raw$y, na.last=NA),]

K <- length(unique(c(dat.raw$t.1, dat.raw$t)))    # Number of treatments
N <- nrow(dat.raw)    # Number of data points

# Add in 95% CIs
dat.raw$CI2.5 <- with(dat.raw, y + qnorm(0.025)*sqrt(Var))
dat.raw$CI97.5 <- with(dat.raw, y + qnorm(0.975)*sqrt(Var))

# Study labels
dat.raw$lab <- with(dat.raw, paste0(studyID," (",t," vs ",t.1,")"))

# Create likelihood covariance matrix
require(Matrix)
V.diag <- as.list(rep(NA,n))
attach(dat.rawl)
for (i in 1:n){
  if (n.arms[i] == 2){
    V.diag[[i]] <- Var.2[i]
  }
  else {
    V.diag[[i]] <- matrix(V[i], nrow=n.arms[i]-1, ncol=n.arms[i]-1)
    tempVar <- c(Var.2[i], Var.3[i], Var.4[i], Var.5[i])
    diag(V.diag[[i]]) <- tempVar[!is.na(tempVar)]
  }
}
detach(dat.rawl)
V <- bdiag(V.diag)

#### STEP 1 ####
# Study level thresholds for "David Clark" effect

# Create contrast "design" matrix
D <- matrix(nrow=K*(K-1)/2, ncol=K-1)
j <- 1
for (i in 1:(K-1)){
  if (i==1) D[j:(j+K-i-1),] <- diag(nrow=K-i)
  else if (i==2) D[j:(j+K-i-1),] <- cbind(rep.int(-1,K-i),diag(nrow=K-i))
  else D[j:(j+K-i-1),] <- cbind(matrix(rep.int(0,(K-i)*(i-2)),ncol=i-2),rep.int(-1,K-i),diag(nrow=K-i))
  j <- j+K-i
}


# Posterior covariance submatrix is
Bstar <- post.cov.ddelta[1:(K-1),K:(K-1+N)]

# Get coefficients [H]_{k*-1,m} - [H]_{a-1,m}
coeffs <- D %*% Bstar %*% solve(V)

# We are only interested in the K-1 (posterior) contrasts that involve k*
kstar <- which(c(0,post.mean)==min(0,post.mean))

# For second place thresholds
k2nd <- which(c(0,post.mean)==sort(c(0,post.mean))[2])

# For CEA treatments only
ceatrt <- c(1, 2, 5, 6, 7, 8, 9, 12, 13, 14, 15, 16, 17, 18, 
            19, 22, 23, 24, 26, 27, 28, 29, 30, 31, 33, 34, 35, 36)
post.mean.subNA <- post.mean
post.mean.subNA[!(1:(K-1) %in% (ceatrt-1))] <- NA
kCEA <- order(c(0,post.mean.subNA), decreasing=FALSE)[1]

# And these contrasts have non-zero elements in the contrast design matrix D in the k* column
contr.kstar <- which(D[,kstar-1] != 0)
contr.k2nd <- which(D[,k2nd-1] != 0)
contr.kCEA <- which(D[,kCEA-1] != 0)[ceatrt[ceatrt!=kCEA] - (ceatrt[ceatrt!=kCEA]>=kCEA)*1]

# So we look in the corresponding rows of the matrix coeffs
coeffs.kstar <- coeffs[contr.kstar,]
coeffs.k2nd <- coeffs[contr.k2nd,]
coeffs.kCEA <- coeffs[contr.kCEA,]

## DAVID CLARK STUDY DATA POINTS ARE ##
dcstud <- which(dat.raw$studyID %in% c(72,79,81))

# How about just bias in trts 33 or 36 (Clark & Wells CT) in Clark studies?
dcstud.ct <- which(dat.raw$studyID %in% c(72,79,81) & 
                     (dat.raw$t %in% c(33,36) | dat.raw$t.1 %in% c(33,36)))


# Get threshold solutions
threshmat <- -(D %*% post.mean)[contr.kstar] / rowSums(coeffs.kstar[,dcstud])
threshmat.ct <- -(D %*% post.mean)[contr.kstar] / rowSums(coeffs.kstar[,dcstud.ct])

threshmat.k2nd <- -(D %*% post.mean)[contr.k2nd] / rowSums(coeffs.k2nd[,dcstud])
threshmat.k2nd.ct <- -(D %*% post.mean)[contr.k2nd] / rowSums(coeffs.k2nd[,dcstud.ct])

threshmat.kCEA <- -(D %*% post.mean)[contr.kCEA] / rowSums(coeffs.kCEA[,dcstud])
threshmat.kCEA.ct <- -(D %*% post.mean)[contr.kCEA] / rowSums(coeffs.kCEA[,dcstud.ct])

# Get positive and negative threshold values, new k*

get.int <- function(x,kstar,trt.sub=1:K){
  if(all(is.infinite(x))){
    hi <- Inf
    lo <- -Inf
    hi.newkstar <- lo.newkstar <- NA
  }
  else if (all(x[!is.infinite(x)]>0)){
    hi <- min(x[!is.infinite(x)])
    hi.newkstar <- trt.sub[which(x==hi) + (which(x==hi) >= which(trt.sub==kstar))*1]
    lo <- -Inf
    lo.newkstar <- NA
  }
  else if (all(x[!is.infinite(x)]<0)){
    hi <- Inf
    hi.newkstar <- NA
    lo <- max(x[!is.infinite(x)])
    lo.newkstar <- trt.sub[which(x==lo) + (which(x==lo) >= which(trt.sub==kstar))*1]
  }
  else {
    hi <- min(x[x>0 & !is.infinite(x)])
    hi.newkstar <- trt.sub[which(x==hi) + (which(x==hi) >= which(trt.sub==kstar))*1]
    lo <- max(x[x<0 & !is.infinite(x)])
    lo.newkstar <- trt.sub[which(x==lo) + (which(x==lo) >= which(trt.sub==kstar))*1]
  }
  return(data.frame(lo=lo,lo.newkstar=lo.newkstar,hi=hi,hi.newkstar=hi.newkstar))
}

thresh <- get.int(threshmat,kstar)
thresh.ct <- get.int(threshmat.ct,kstar)

thresh.k2nd <- get.int(threshmat.k2nd,k2nd)
thresh.k2nd.ct <- get.int(threshmat.k2nd.ct,k2nd)

thresh.kCEA <- get.int(threshmat.kCEA,kCEA,ceatrt)
thresh.kCEA.ct <- get.int(threshmat.kCEA.ct,kCEA,ceatrt)

## Plot
plotme <- function(thresh, contr.mean, CrI.lo, CrI.hi, rowlabs, xlim, xlab, ylab, ...){
  
  yaxs <- length(contr.mean):1
  
  # split plot in two
  layout(matrix(1:2,nrow=1), widths=c(.2,1))
  
  # plot row labels on left side
  gp <- par(mar=c(5,4,1,0))
  plot(rep(0,length(yaxs)), yaxs, pch=NA, ylim=c(.5,yaxs[1]+.5), ylab="",
       xlab="", yaxt="n", xaxt="n", bty="n")
  text(0, yaxs, labels=rowlabs,xpd=NA)
  title(ylab=ylab, line=2)
  
  # fake plot for setup of right side
  par(mar=c(5,1,1,2))
  plot(contr.mean, yaxs, pch=NA, yaxt="n", xlim=xlim,
       ylim=c(.5,yaxs[1]+.5), ylab="", xlab="",...)
  title(xlab=xlab, line=2)
  
  # reference line
  abline(v=0, lty=2, col="gray")
  
  # combined invariant region
  smallrgb <- rgb(.9,.7,.8,.8)
  smallrgb2 <- rgb(.9,.7,.8)
  largergb <- rgb(.7,.8,.9,.7)
  largergb2 <- rgb(.7,.8,.9)
  
  # fix infinite thresholds
  if(is.infinite(thresh$lo)) polylo <- xlim[1]*2
  else polylo <- thresh$lo
  
  if(is.infinite(thresh$hi)) polyhi <- xlim[2]*2
  else polyhi <- thresh$hi
  
  # lo
  polygon(rep(c(contr.mean + polylo, rev(contr.mean)), each=2), 
          c(rep(yaxs,each=2) + c(.5,-.5), rep(rev(yaxs),each=2) + c(-.5,.5)),
          col=ifelse(all(CrI.lo <= contr.mean + thresh$lo), smallrgb, largergb),
          border=ifelse(all(CrI.lo <= contr.mean + thresh$lo), smallrgb2, largergb2))
  
  # hi
  polygon(rep(c(contr.mean, rev(contr.mean) + polyhi), each=2), 
          c(rep(yaxs,each=2) + c(.5,-.5), rep(rev(yaxs),each=2) + c(-.5,.5)),
          col=ifelse(all(CrI.hi >= contr.mean + thresh$hi), smallrgb, largergb),
          border=ifelse(all(CrI.hi >= contr.mean + thresh$hi), smallrgb2, largergb2))
  
  # credible intervals
  segments(y0=yaxs, x0=CrI.lo, x1=CrI.hi, lend=1)
  
  # contrast means
  points(contr.mean, yaxs, pch=21, col="black", bg="white")
  
  # deal with infinite thresholds
  # write new optimal treatments at either side
  if(!is.na(thresh$lo.newkstar)){
    text(xlim[1], round(length(yaxs)/2), 
         labels=as.expression(substitute(tilde(k)*"* = "*xx,list(xx=thresh$lo.newkstar))), 
         pos=4)
  }
  if(!is.na(thresh$hi.newkstar)){
    text(xlim[2], round(length(yaxs)/2),
         labels=as.expression(substitute(tilde(k)*"* = "*xx,list(xx=thresh$hi.newkstar))),
         pos=2)
  }
  
  # write invariant interval below plot
  if(is.infinite(thresh$lo)){
    lo.thr <- "NT"
    lo.k <- "-"
  } else {
    lo.thr <- formatC(thresh$lo,digits=2,format="f")
    lo.k <- thresh$lo.newkstar
  }
  if(is.infinite(thresh$hi)){
    hi.thr <- "NT"
    hi.k <- "-"
  } else {
    hi.thr <- formatC(thresh$hi,digits=2,format="f")
    hi.k <- thresh$hi.newkstar
  }
  
  title(xlab=paste0("Invariant interval: ",lo.k," (",lo.thr,", ",hi.thr,") ",hi.k), line=3)
  
  par(gp)
}

pdf("Social Anxiety thresholds - study level Clark effect.pdf", width=8, height=3)
with(dat.raw[dcstud,],
  plotme(thresh, y, CI2.5, CI97.5,
         paste0(studyID,": ",t," vs. ",t.1),
         xlim=c(-4,3), ylab="Clark study estimates", xlab="SMD")
)
dev.off()

pdf("Social Anxiety thresholds - study level Clark effect CT ONLY.pdf", width=8, height=3)
with(dat.raw[dcstud.ct,],
     plotme(thresh.ct, y, CI2.5, CI97.5,
            paste0(studyID,": ",t," vs. ",t.1),
            xlim=c(-4,3), ylab="Clark study estimates", xlab="SMD")
)
dev.off()

pdf("Social Anxiety thresholds - study level Clark effect - second place.pdf", width=8, height=3)
with(dat.raw[dcstud,],
     plotme(thresh.k2nd, y, CI2.5, CI97.5,
            paste0(studyID,": ",t," vs. ",t.1),
            xlim=c(-4,3), ylab="Clark study estimates", xlab="SMD")
)
dev.off()

pdf("Social Anxiety thresholds - study level Clark effect CT ONLY - second place.pdf", 
    width=8, height=3)
with(dat.raw[dcstud.ct,],
     plotme(thresh.k2nd.ct, y, CI2.5, CI97.5,
            paste0(studyID,": ",t," vs. ",t.1),
            xlim=c(-4,3), ylab="Clark study estimates", xlab="SMD")
)
dev.off()

pdf("Social Anxiety thresholds - study level Clark effect - CEA treatments.pdf", width=8, height=3)
with(dat.raw[dcstud,],
     plotme(thresh.kCEA, y, CI2.5, CI97.5,
            paste0(studyID,": ",t," vs. ",t.1),
            xlim=c(-4,1.1), ylab="Clark study estimates", xlab="SMD")
)
dev.off()

pdf("Social Anxiety thresholds - study level Clark effect CT ONLY - CEA treatments.pdf", 
    width=8, height=3)
with(dat.raw[dcstud.ct,],
     plotme(thresh.kCEA.ct, y, CI2.5, CI97.5,
            paste0(studyID,": ",t," vs. ",t.1),
            xlim=c(-4,1), ylab="Clark study estimates", xlab="SMD")
)
dev.off()


### CT study estimates
# ctstud <- which(dat.raw$t %in% c(33,36) | dat.raw$t.1 %in% c(33,36))
# 
# plotme(NULL,contr.mean=dat.raw[ctstud,"y"], CrI.lo=dat.raw[ctstud,"CI2.5"],
#        CrI.hi=dat.raw[ctstud,"CI97.5"], rowlabs="")
# 
# plot(dat.raw[ctstud,"y"],length(ctstud):1, pch=20+(ctstud %in% dcstud.ct), xlim=c(-3,1))
# segments(y0=length(ctstud):1, x0=dat.raw[ctstud,"CI2.5"], x1=dat.raw[ctstud,"CI97.5"], lend=1)
# abline(v=0, lty=2, col="gray")


#### BIAS IN ALL DRUG COMPARISONS WITH INACTIVE ####

# Get denominators for each threshold equation by summing the coefficients of
# all DRUG data points. Do this with and without combined regimens.

# Drug treatments
drugtrts <- 9:23
drugtrts.withcombined <- c(9:23, 37:41)

# Which data points compare these to an inactive trt?
drugdats <- which(dat.raw$t.1 %in% 1:3 & dat.raw$t %in% drugtrts)
drugdats.withcombined <- which(dat.raw$t.1 %in% 1:3 & dat.raw$t %in% drugtrts.withcombined)

# Get threshold solutions
threshmat.drugs <- -(D %*% post.mean)[contr.kstar] / rowSums(coeffs.kstar[,drugdats])
threshmat.drugs.withcombined <- -(D %*% post.mean)[contr.kstar] / rowSums(coeffs.kstar[,drugdats.withcombined])

threshmat.k2nd.drugs <- -(D %*% post.mean)[contr.k2nd] / rowSums(coeffs.k2nd[,drugdats])
threshmat.k2nd.drugs.withcombined <- -(D %*% post.mean)[contr.k2nd] / rowSums(coeffs.k2nd[,drugdats.withcombined])

threshmat.ceatrt.drugs <- -(D %*% post.mean)[contr.kCEA] / rowSums(coeffs.kCEA[,drugdats])
threshmat.ceatrt.drugs.withcombined <- -(D %*% post.mean)[contr.kCEA] / rowSums(coeffs.kCEA[,drugdats.withcombined])


# Get positive and negative threshold values, new k*
thresh.drugs <- get.int(threshmat.drugs,kstar)
thresh.drugs.withcombined <- get.int(threshmat.drugs.withcombined,kstar)

thresh.k2nd.drugs <- get.int(threshmat.k2nd.drugs,k2nd)
thresh.k2nd.drugs.withcombined <- get.int(threshmat.k2nd.drugs.withcombined,k2nd)

thresh.ceatrt.drugs <- get.int(threshmat.ceatrt.drugs, kCEA, ceatrt)
thresh.ceatrt.drugs.withcombined <- get.int(threshmat.ceatrt.drugs.withcombined, kCEA, ceatrt)


pdf("Social Anxiety thresholds - study level by group - drugs.pdf", width=9, height=10)
plotme(thresh.drugs, 
       dd.summary$stat[drugdats,"Mean"], 
       dd.summary$quant[drugdats,"2.5%"],
       dd.summary$quant[drugdats,"97.5%"],
       rowlabs=paste0(dat.raw[drugdats,"studyID"]," (",
                     dat.raw[drugdats,"t"]," vs. ",dat.raw[drugdats,"t.1"],")"),
       xlim=c(-3,2.5), ylab="Drug vs. Inactive Study Estimates", xlab="SMD")
dev.off()

pdf("Social Anxiety thresholds - study level by group - drugs with combined.pdf", width=9, height=10)
plotme(thresh.drugs.withcombined, 
       dd.summary$stat[drugdats.withcombined,"Mean"], 
       dd.summary$quant[drugdats.withcombined,"2.5%"],
       dd.summary$quant[drugdats.withcombined,"97.5%"],
       rowlabs=paste0(dat.raw[drugdats.withcombined,"studyID"]," (",
                      dat.raw[drugdats.withcombined,"t"]," vs. ",dat.raw[drugdats.withcombined,"t.1"],")"),
       xlim=c(-4,1), ylab="Drug vs. Inactive Study Estimates", xlab="SMD")
dev.off()

pdf("Social Anxiety thresholds - study level by group - drugs - second place.pdf", width=9, height=10)
plotme(thresh.k2nd.drugs, 
       dd.summary$stat[drugdats,"Mean"], 
       dd.summary$quant[drugdats,"2.5%"],
       dd.summary$quant[drugdats,"97.5%"],
       rowlabs=paste0(dat.raw[drugdats,"studyID"]," (",
                      dat.raw[drugdats,"t"]," vs. ",dat.raw[drugdats,"t.1"],")"),
       xlim=c(-3,2.5), ylab="Drug vs. Inactive Study Estimates", xlab="SMD")
dev.off()

pdf("Social Anxiety thresholds - study level by group - drugs with combined - second place.pdf", width=9, height=10)
plotme(thresh.k2nd.drugs.withcombined, 
       dd.summary$stat[drugdats.withcombined,"Mean"], 
       dd.summary$quant[drugdats.withcombined,"2.5%"],
       dd.summary$quant[drugdats.withcombined,"97.5%"],
       rowlabs=paste0(dat.raw[drugdats.withcombined,"studyID"]," (",
                      dat.raw[drugdats.withcombined,"t"]," vs. ",dat.raw[drugdats.withcombined,"t.1"],")"),
       xlim=c(-4,1), ylab="Drug vs. Inactive Study Estimates", xlab="SMD")
dev.off()

pdf("Social Anxiety thresholds - study level by group - drugs - CEA treatments.pdf", 
    width=9, height=10)
plotme(thresh.ceatrt.drugs, 
       dd.summary$stat[drugdats,"Mean"], 
       dd.summary$quant[drugdats,"2.5%"],
       dd.summary$quant[drugdats,"97.5%"],
       rowlabs=paste0(dat.raw[drugdats,"studyID"]," (",
                      dat.raw[drugdats,"t"]," vs. ",dat.raw[drugdats,"t.1"],")"),
       xlim=c(-3,6), ylab="Drug vs. Inactive Study Estimates", xlab="SMD")
dev.off()

pdf("Social Anxiety thresholds - study level by group - drugs with combined - CEA treatments.pdf", 
    width=9, height=10)
plotme(thresh.ceatrt.drugs.withcombined, 
       dd.summary$stat[drugdats.withcombined,"Mean"], 
       dd.summary$quant[drugdats.withcombined,"2.5%"],
       dd.summary$quant[drugdats.withcombined,"97.5%"],
       rowlabs=paste0(dat.raw[drugdats.withcombined,"studyID"]," (",
                      dat.raw[drugdats.withcombined,"t"]," vs. ",dat.raw[drugdats.withcombined,"t.1"],")"),
       xlim=c(-3,5), ylab="Drug vs. Inactive Contrasts", xlab="SMD")
dev.off()

#### BIAS IN ALL PSYCH COMPARISONS WITH INACTIVE ####

# Get denominators for each threshold equation by summing the coefficients of
# all PSYCH data points. Do this with and without combined regimens.

# Psych treatments
psychtrts <- c(4:8, 24:36)
psychtrts.withcombined <- c(4:8, 24:36, 37:41)

# Which data points compare these to an inactive trt?
psychdats <- which(dat.raw$t.1 %in% 1:3 & dat.raw$t %in% psychtrts)
psychdats.withcombined <- which(dat.raw$t.1 %in% 1:3 & dat.raw$t %in% psychtrts.withcombined)

# Get threshold solutions
threshmat.psych <- -(D %*% post.mean)[contr.kstar] / rowSums(coeffs.kstar[,psychdats])
threshmat.psych.withcombined <- -(D %*% post.mean)[contr.kstar] / rowSums(coeffs.kstar[,psychdats.withcombined])

threshmat.k2nd.psych <- -(D %*% post.mean)[contr.k2nd] / rowSums(coeffs.k2nd[,psychdats])
threshmat.k2nd.psych.withcombined <- -(D %*% post.mean)[contr.k2nd] / rowSums(coeffs.k2nd[,psychdats.withcombined])

threshmat.ceatrt.psych <- -(D %*% post.mean)[contr.kCEA] / rowSums(coeffs.kCEA[,psychdats])
threshmat.ceatrt.psych.withcombined <- -(D %*% post.mean)[contr.kCEA] / rowSums(coeffs.kCEA[,psychdats.withcombined])


# Get positive and negative threshold values, new k*
thresh.psych <- get.int(threshmat.psych,kstar)
thresh.psych.withcombined <- get.int(threshmat.psych.withcombined,kstar)

thresh.k2nd.psych <- get.int(threshmat.k2nd.psych,k2nd)
thresh.k2nd.psych.withcombined <- get.int(threshmat.k2nd.psych.withcombined,k2nd)

thresh.ceatrt.psych <- get.int(threshmat.ceatrt.psych, kCEA, ceatrt)
thresh.ceatrt.psych.withcombined <- get.int(threshmat.ceatrt.psych.withcombined, kCEA, ceatrt)

## Plot
pdf("Social Anxiety thresholds - study level by group - psych.pdf", width=9, height=10)
plotme(thresh.psych, 
       dd.summary$stat[psychdats,"Mean"], 
       dd.summary$quant[psychdats,"2.5%"],
       dd.summary$quant[psychdats,"97.5%"],
       rowlabs=paste0(dat.raw[psychdats,"studyID"]," (",
                      dat.raw[psychdats,"t"]," vs. ",dat.raw[psychdats,"t.1"],")"),
       xlim=c(-3,3), ylab="Psych vs. Inactive Study Estimates", xlab="SMD")
dev.off()

pdf("Social Anxiety thresholds - study level by group - psych with combined.pdf", width=9, height=10)
plotme(thresh.psych.withcombined, 
       dd.summary$stat[psychdats.withcombined,"Mean"], 
       dd.summary$quant[psychdats.withcombined,"2.5%"],
       dd.summary$quant[psychdats.withcombined,"97.5%"],
       rowlabs=paste0(dat.raw[psychdats.withcombined,"studyID"]," (",
                      dat.raw[psychdats.withcombined,"t"]," vs. ",dat.raw[psychdats.withcombined,"t.1"],")"),
       xlim=c(-7,2), ylab="Psych vs. Inactive Study Estimates", xlab="SMD")
dev.off()

pdf("Social Anxiety thresholds - study level by group - psych - second place.pdf", width=9, height=10)
plotme(thresh.k2nd.psych, 
       dd.summary$stat[psychdats,"Mean"], 
       dd.summary$quant[psychdats,"2.5%"],
       dd.summary$quant[psychdats,"97.5%"],
       rowlabs=paste0(dat.raw[psychdats,"studyID"]," (",
                      dat.raw[psychdats,"t"]," vs. ",dat.raw[psychdats,"t.1"],")"),
       xlim=c(-2,1.5), ylab="Psych vs. Inactive Study Estimates", xlab="SMD")
dev.off()

pdf("Social Anxiety thresholds - study level by group - psych with combined - second place.pdf", width=9, height=10)
plotme(thresh.k2nd.psych.withcombined, 
       dd.summary$stat[psychdats.withcombined,"Mean"], 
       dd.summary$quant[psychdats.withcombined,"2.5%"],
       dd.summary$quant[psychdats.withcombined,"97.5%"],
       rowlabs=paste0(dat.raw[psychdats.withcombined,"studyID"]," (",
                      dat.raw[psychdats.withcombined,"t"]," vs. ",dat.raw[psychdats.withcombined,"t.1"],")"),
       xlim=c(-6,2), ylab="Psych vs. Inactive Study Estimates", xlab="SMD")
dev.off()

pdf("Social Anxiety thresholds - study level by group - psych - CEA treatments.pdf", 
    width=9, height=10)
plotme(thresh.ceatrt.psych, 
       dd.summary$stat[psychdats,"Mean"], 
       dd.summary$quant[psychdats,"2.5%"],
       dd.summary$quant[psychdats,"97.5%"],
       rowlabs=paste0(dat.raw[psychdats,"studyID"]," (",
                      dat.raw[psychdats,"t"]," vs. ",dat.raw[psychdats,"t.1"],")"),
       xlim=c(-5,2), ylab="Psych vs. Inactive Study Estimates", xlab="SMD")
dev.off()

pdf("Social Anxiety thresholds - study level by group - psych with combined - CEA treatments.pdf", 
    width=9, height=10)
plotme(thresh.ceatrt.psych.withcombined, 
       dd.summary$stat[psychdats.withcombined,"Mean"], 
       dd.summary$quant[psychdats.withcombined,"2.5%"],
       dd.summary$quant[psychdats.withcombined,"97.5%"],
       rowlabs=paste0(dat.raw[psychdats.withcombined,"studyID"]," (",
                      dat.raw[psychdats.withcombined,"t"]," vs. ",dat.raw[psychdats.withcombined,"t.1"],")"),
       xlim=c(-5,2), ylab="Psych vs. Inactive Study Estimates", xlab="SMD")
dev.off()
