############################################################
# CG159 Social Anxiety: Study Level Threshold Analysis
#   - Performed on standardised mean difference scale (SMD)
############################################################

source("../../../Functions/nma_threshRE.R")
source("../../../Functions/thresh_forest.R")

#### STEP 0 ####
# Read CODA data
source("./Social Anxiety - read CODA.R")

# Read study data
dat.rawl <- read.table(file = "../WinBUGS/study data.txt", header = TRUE, sep = "\t")

n <- nrow(dat.rawl)   # Number of studies

# Turn wide study data into long with one row for each arm
dat.raw <- reshape(dat.rawl, varying=c("t.2","y.2","Var.2","t.3","y.3","Var.3",
                                      "t.4","y.4","Var.4","t.5","y.5","Var.5"), 
                   timevar="arm", idvar="studyID", direction="long")

# Sort data by study and contrast, removing NA rows
dat.raw <- dat.raw[order(dat.raw$studyID, dat.raw$arm, dat.raw$y, na.last=NA),]

K <- length(unique(c(dat.raw$t.1, dat.raw$t)))    # Number of treatments
N <- nrow(dat.raw)    # Number of data points

# Add in 95% CIs
dat.raw$CI2.5 <- with(dat.raw, y + qnorm(0.025)*sqrt(Var))
dat.raw$CI97.5 <- with(dat.raw, y + qnorm(0.975)*sqrt(Var))

# Study labels
dat.raw$lab <- with(dat.raw, paste0(studyID," (",t," vs ",t.1,")"))

# Create likelihood covariance matrix
require(Matrix)
V.diag <- as.list(rep(NA,n))
attach(dat.rawl)
for (i in 1:n){
  if (n.arms[i] == 2){
    V.diag[[i]] <- Var.2[i]
  }
  else {
    V.diag[[i]] <- matrix(V[i], nrow=n.arms[i]-1, ncol=n.arms[i]-1)
    tempVar <- c(Var.2[i], Var.3[i], Var.4[i], Var.5[i])
    diag(V.diag[[i]]) <- tempVar[!is.na(tempVar)]
  }
}
detach(dat.rawl)
V <- bdiag(V.diag)

##### STEP 1 #####
# Derive thresholds
thresh <- nma_threshRE(post.mean, V, post.cov.ddelta, opt.max=F)

# Merge with data
thresh <- cbind(dat.raw, thresh)

# Create threshold plots
## COLOUR
pdf("Social Anxiety thresholds - study level.pdf", width=13, height=30)
with(thresh,
     thresh_forest(means=y, t.lo=lo, t.hi=hi, kstar.lo=lo.newkstar, kstar.hi=hi.newkstar,
                   CI.lo=CI2.5, CI.hi=CI97.5, label=lab, label.title="Study (Contrast)",
                   CI.title="95% Confidence Interval",
                   xlab="Standardised Mean Difference", xlim=c(-4,3), mean.title="SMD",
                   refline=0, digits=2)
)
dev.off()

## B&W
pdf("Social Anxiety thresholds - study level BW.pdf", width=13, height=30)
with(thresh,
     thresh_forest(means=y, t.lo=lo, t.hi=hi, kstar.lo=lo.newkstar, kstar.hi=hi.newkstar,
                   CI.lo=CI2.5, CI.hi=CI97.5, label=lab, label.title="Study (Contrast)",
                   CI.title="95% Confidence Interval",
                   xlab="Standardised Mean Difference", xlim=c(-4,3), mean.title="SMD",
                   refline=0, digits=2, greyscale=TRUE)
)
dev.off()

# Sorted by contrast
thresh.sort <- thresh[order(thresh$t.1, thresh$t, thresh$studyID),]

## COLOUR
pdf("Social Anxiety thresholds - study level - sorted.pdf", width=13, height=30)
with(thresh.sort,
     thresh_forest(means=y, t.lo=lo, t.hi=hi, kstar.lo=lo.newkstar, kstar.hi=hi.newkstar,
                   CI.lo=CI2.5, CI.hi=CI97.5, label=lab, label.title="Study (Contrast)",
                   CI.title="95% Confidence Interval",
                   xlab="Standardised Mean Difference", xlim=c(-4,3), mean.title="SMD",
                   refline=0, digits=2)
)
dev.off()

## B&W
pdf("Social Anxiety thresholds - study level - sorted BW.pdf", width=13, height=30)
with(thresh.sort,
     thresh_forest(means=y, t.lo=lo, t.hi=hi, kstar.lo=lo.newkstar, kstar.hi=hi.newkstar,
                   CI.lo=CI2.5, CI.hi=CI97.5, label=lab, label.title="Study (Contrast)",
                   CI.title="95% Confidence Interval",
                   xlab="Standardised Mean Difference", xlim=c(-4,3), mean.title="SMD",
                   refline=0, digits=2, greyscale=TRUE)
)
dev.off()

## Only report contrasts where the threshold is less than 3 SMD
cutoff <- 3
thresh.cut <- thresh[thresh$lo > -cutoff | thresh$hi < cutoff, ]

## COLOUR
pdf("Social Anxiety thresholds - study level CUTOFF.pdf", width=13, height=9.5)
with(thresh.cut,
     thresh_forest(means=y, t.lo=lo, t.hi=hi, kstar.lo=lo.newkstar, kstar.hi=hi.newkstar,
                   CI.lo=CI2.5, CI.hi=CI97.5, label=lab, label.title="Study (Contrast)",
                   CI.title="95% Confidence Interval",
                   xlab="Standardised Mean Difference", xlim=c(-4,3), mean.title="SMD",
                   refline=0, digits=2)
)
dev.off()

## Sorted with smallest thresholds first
absmin <- function(x) min(abs(x))
thresh.cut.sort <- thresh.cut[order(apply(thresh.cut[,c("lo","hi")], 1, absmin)),]

## COLOUR
pdf("Social Anxiety thresholds - study level CUTOFF SORT.pdf", width=13, height=9.5)
with(thresh.cut.sort,
     thresh_forest(means=y, t.lo=lo, t.hi=hi, kstar.lo=lo.newkstar, kstar.hi=hi.newkstar,
                   CI.lo=CI2.5, CI.hi=CI97.5, label=lab, label.title="Study (Contrast)",
                   CI.title="95% Confidence Interval",
                   xlab="Standardised Mean Difference", mean.title="SMD", xlim=c(-4,3), 
                   refline=0, digits=2)
)
dev.off()

##### Thresholds for second place #####

# Get thresholds
#thresh.k2nd <- as.data.frame(do.call(rbind, apply(threshmat.k2nd, 2, get.int, kstar=k2nd)))
thresh.k2nd <- nma_threshRE(post.mean, V, post.cov.ddelta, trt.rank=2, opt.max=F)

# Merge with data
thresh.k2nd <- cbind(dat.raw, thresh.k2nd)

# Create threshold plots
## COLOUR
pdf("Social Anxiety thresholds - study level - second place.pdf", width=13, height=30)
with(thresh.k2nd,
     thresh_forest(means=y, t.lo=lo, t.hi=hi, kstar.lo=lo.newkstar, kstar.hi=hi.newkstar,
                   CI.lo=CI2.5, CI.hi=CI97.5, label=lab, label.title="Study (Contrast)",
                   CI.title="95% Confidence Interval",
                   xlab="Standardised Mean Difference", xlim=c(-4,3), mean.title="SMD",
                   refline=0, digits=2)
)
dev.off()

## Only report contrasts where the threshold is less than 3 SMD
thresh.k2nd.cut <- thresh.k2nd[thresh.k2nd$lo > -cutoff | thresh.k2nd$hi < cutoff, ]

## COLOUR
pdf("Social Anxiety thresholds - study level - second place CUTOFF.pdf", width=13, height=10.5)
with(thresh.k2nd.cut,
     thresh_forest(means=y, t.lo=lo, t.hi=hi, kstar.lo=lo.newkstar, kstar.hi=hi.newkstar,
                   CI.lo=CI2.5, CI.hi=CI97.5, label=lab, label.title="Study (Contrast)",
                   CI.title="95% Confidence Interval",
                   xlab="Standardised Mean Difference", xlim=c(-4,3), mean.title="SMD",
                   refline=0, digits=2)
)
dev.off()

## Sorted with smallest thresholds first
thresh.k2nd.cut.sort <- thresh.k2nd.cut[order(apply(thresh.k2nd.cut[,c("lo","hi")], 1, absmin)),]

## COLOUR
pdf("Social Anxiety thresholds - study level - second place CUTOFF SORT.pdf", width=13, height=10.5)
with(thresh.k2nd.cut.sort,
     thresh_forest(means=y, t.lo=lo, t.hi=hi, kstar.lo=lo.newkstar, kstar.hi=hi.newkstar,
                   CI.lo=CI2.5, CI.hi=CI97.5, label=lab, label.title="Study (Contrast)",
                   CI.title="95% Confidence Interval",
                   xlab="Standardised Mean Difference", mean.title="SMD", xlim=c(-4,3), 
                   refline=0, digits=2)
)
dev.off()

##### Thresholds for CEA treatments only #####

# Get thresholds
ceatrt <- c(1, 2, 5, 6, 7, 8, 9, 12, 13, 14, 15, 16, 17, 18, 
            19, 22, 23, 24, 26, 27, 28, 29, 30, 31, 33, 34, 35, 36)
thresh.ceatrt <- nma_threshRE(post.mean, V, post.cov.ddelta, trt.sub=ceatrt, opt.max=F)

# Merge with data
thresh.ceatrt <- cbind(dat.raw, thresh.ceatrt)

# Create threshold plots
## COLOUR
pdf("Social Anxiety thresholds - study level - CEA treatments.pdf", width=13, height=30)
with(thresh.ceatrt,
     thresh_forest(means=y, t.lo=lo, t.hi=hi, kstar.lo=lo.newkstar, kstar.hi=hi.newkstar,
                   CI.lo=CI2.5, CI.hi=CI97.5, label=lab, label.title="Study (Contrast)",
                   CI.title="95% Confidence Interval",
                   xlab="Standardised Mean Difference", xlim=c(-4,3), mean.title="SMD",
                   refline=0, digits=2)
)
dev.off()

## Only report contrasts where the threshold is less than 3 SMD
thresh.ceatrt.cut <- thresh.ceatrt[thresh.ceatrt$lo > -cutoff | thresh.ceatrt$hi < cutoff, ]

## COLOUR
pdf("Social Anxiety thresholds - study level - CEA treatments CUTOFF.pdf", width=13, height=10.5)
with(thresh.ceatrt.cut,
     thresh_forest(means=y, t.lo=lo, t.hi=hi, kstar.lo=lo.newkstar, kstar.hi=hi.newkstar,
                   CI.lo=CI2.5, CI.hi=CI97.5, label=lab, label.title="Study (Contrast)",
                   CI.title="95% Confidence Interval",
                   xlab="Standardised Mean Difference", xlim=c(-4,3), mean.title="SMD",
                   refline=0, digits=2)
)
dev.off()

## Sorted with smallest thresholds first
thresh.ceatrt.cut.sort <- thresh.ceatrt.cut[order(apply(thresh.ceatrt.cut[,c("lo","hi")], 1, absmin)),]

## COLOUR
pdf("Social Anxiety thresholds - study level - CEA treatments CUTOFF SORT.pdf", width=13, height=10.5)
with(thresh.ceatrt.cut.sort,
     thresh_forest(means=y, t.lo=lo, t.hi=hi, kstar.lo=lo.newkstar, kstar.hi=hi.newkstar,
                   CI.lo=CI2.5, CI.hi=CI97.5, label=lab, label.title="Study (Contrast)",
                   CI.title="95% Confidence Interval",
                   xlab="Standardised Mean Difference", mean.title="SMD", xlim=c(-4,3), 
                   refline=0, digits=2)
)
dev.off()

