########################################################
# CG171 Urinary Incontinence: Contrast level analysis
########################################################

source("../../../Functions/recon_vcov.R")
source("../../../Functions/nma_thresh.R")
source("../../../Functions/thresh_forest.R")
source("../../../Functions/d_ab2i.R")


#### STEP 0 ####
# Read CODA data
source("./UI - read CODA.R")


#### STEP 1 ####
# First, we must reconstruct the likelihood covariance matrix.

# Create contrast design matrix X. We'll use the treatment details from the study 
# level data for convenience.

trt.dat <- read.table(file = "../WinBUGS/treatment details.txt", header = TRUE, sep = "\t")
K <- with(trt.dat, length(unique(c(t.1, t.2, t.3)))) -1  # Number of treatments , -1 for NA

# work out which contrasts have data
contr.ab <- data.frame(a=c(), b=c())

for(i in 1:nrow(trt.dat)){
  rowi <- trt.dat[i,which(!is.na(trt.dat[i,2:4]))+1] # non NA elements of ith row
  
  # get contrast from all combinations of treatments
  trtcomb <- combn(rowi, 2, function(x) sapply(x, as.numeric))
  
  a <- apply(trtcomb, 2, min)
  b <- apply(trtcomb, 2, max)
  
  # remove contrasts of treatments against themselves
  iseq <- a==b
  a <- a[!iseq]
  b <- b[!iseq]
  
  if (!all(iseq)) contr.ab <- rbind(contr.ab, cbind(a,b))
}

contr.ab <- unique(contr.ab[order(contr.ab$a, contr.ab$b),])


# Contrast design matrix
X <- matrix(0, nrow=nrow(contr.ab), ncol=K-1)
for (i in 1:nrow(X)){
  if (contr.ab[i,"a"] > 1) X[i, contr.ab[i,"a"] - 1]  <- -1
  if (contr.ab[i,"b"] > 1)   X[i, contr.ab[i,"b"] - 1]    <- 1
}

# Reconstruct using NNLS
lik.cov <- recon_vcov(post.cov, prior.prec=.0001, X=X)

# Get indices of contrasts in likelihood in full contrast vector
d.a <- d.b <- vector(length=nrow(X))
for (i in 1:nrow(X)){
  d.a[i] <- ifelse(any(X[i,]==-1),which(X[i,]==-1),0)+1
  d.b[i] <- ifelse(any(X[i,]==1),which(X[i,]==1),0)+1
}

d.i <- d_ab2i(d.a, d.b, K=K)

#### STEP 2 ####
# Now we can perform thresholding at the contrast level.

# Thresholds are
thresh <- nma_thresh(X, d.mean, lik.cov, post.cov, opt.max=TRUE)

# Label the contrasts and display using a forest plot, along with 95% posterior
# credible intervals as reported in CG150.1

#contr <- paste0("d[",d.a,d.b,"]")
contr <- paste0(d.b," vs. ",d.a)

contr.mean <- dd.mean[d.i]
CI2.5 <- dd.summary$quant[d.i,"2.5%"]
CI97.5 <- dd.summary$quant[d.i,"97.5%"]
print(thresh <- as.data.frame(cbind(contr,contr.mean,CI2.5,CI97.5,thresh)), row.names=FALSE)

## COLOUR
pdf("UI thresholds - contrast level.pdf", width=13, height=6)
with(thresh,
     thresh_forest(means=contr.mean, t.lo=lo, t.hi=hi, kstar.lo=lo.newkstar, kstar.hi=hi.newkstar,
                   CI.lo=CI2.5, CI.hi=CI97.5, label=contr, label.title="Contrast",
                   xlab="Log Odds Ratio", xlim=c(-2,1.5), 
                   refline=0, digits=2)
)
dev.off()

## B&W
pdf("UI thresholds - contrast level BW.pdf", width=13, height=6)
with(thresh,
     thresh_forest(means=contr.mean, t.lo=lo, t.hi=hi, kstar.lo=lo.newkstar, kstar.hi=hi.newkstar,
                   CI.lo=CI2.5, CI.hi=CI97.5, label=contr, label.title="Contrast",
                   xlab="Log Odds Ratio", xlim=c(-2,1.5), 
                   refline=0, digits=2, cutoff=25, greyscale=TRUE)
)
dev.off()

## Thresholds for choosing placebo first
thresh.pbo <- nma_thresh(X, d.mean, lik.cov, post.cov, opt.max=TRUE, trt.sub=c(1,14))
print(thresh.pbo <- as.data.frame(cbind(contr,contr.mean,CI2.5,CI97.5,thresh.pbo)), row.names=FALSE)

## COLOUR
pdf("UI thresholds - contrast level - placebo.pdf", width=13, height=6)
with(thresh.pbo,
     thresh_forest(means=contr.mean, t.lo=lo, t.hi=hi, kstar.lo=lo.newkstar, kstar.hi=hi.newkstar,
                   CI.lo=CI2.5, CI.hi=CI97.5, label=contr, label.title="Contrast",
                   xlab="Log Odds Ratio", xlim=c(-2,1.5), 
                   refline=0, digits=2)
)
dev.off()

