########################################################
# CG171 Urinary Incontinence: Study level analysis
########################################################

source("../../../Functions/nma_thresh.R")
source("../../../Functions/thresh_forest.R")
require(Matrix)

#### STEP 0 ####

# Read in the posterior data, as generated from WinBUGS.
source("./UI - read CODA.R")

# Read in the study data
dat <- read.delim("../WinBUGS/study data.txt")

# Log odds ratios between arms
dat$lor1.1 <- with(dat, log(r1.2 * (n.1 - r1.1) / ((n.2 - r1.2) * r1.1)) )
dat$lor1.2 <- with(dat, log(r1.3 * (n.1 - r1.1) / ((n.3 - r1.3) * r1.1)) )
dat$lor2.1 <- with(dat, log(r2.2 * (n.1 - r2.1) / ((n.2 - r2.2) * r2.1)) )
dat$lor2.2 <- with(dat, log(r2.3 * (n.1 - r2.1) / ((n.3 - r2.3) * r2.1)) )

# Record which was treatment and which was baseline for each LOR
dat$k.1 <- dat$t.2
dat$b.1 <- dat$t.1
dat$k.2 <- dat$t.3
dat$b.2 <- ifelse(is.na(dat$k.2),NA,dat$t.1)

# LOR variances and covariances, likelihood covariance matrix V
V.diag <- as.list(rep(NA,nrow(dat)))
attach(dat)
for (i in 1:nrow(dat)){
  if (n.timept[i] == 1){
    if (n.arms[i] == 2){
      V.diag[[i]] <- 1/r1.1[i] + 1/r1.2[i] + 1/(n.1[i]-r1.1[i]) + 1/(n.2[i]-r1.2[i])
    }
    else if (n.arms[i] == 3){
      v1 <- 1/r1.1[i] + 1/r1.2[i] + 1/(n.1[i]-r1.1[i]) + 1/(n.2[i]-r1.2[i])
      v2 <- 1/r1.1[i] + 1/r1.3[i] + 1/(n.1[i]-r1.1[i]) + 1/(n.3[i]-r1.3[i])
      # Covariance term Eqn 10 from Bagos (2012)
      c1 <- 1/r1.1[i] + 1/(n.1[i] - r1.1[i])
      V.diag[[i]] <- matrix(c(v1,c1,c1,v2),nrow=2)
    }
  }
  else if (n.timept[i] == 2){
    # Assuming that LORs are independent across time
    if (n.arms[i] == 2){
      V.diag[[i]] <- diag(c(1/r1.1[i] + 1/r1.2[i] + 1/(n.1[i]-r1.1[i]) + 1/(n.2[i]-r1.2[i]),
                          1/r2.1[i] + 1/r2.2[i] + 1/(n.1[i]-r2.1[i]) + 1/(n.2[i]-r2.2[i])))
    }
    else if (n.arms[i] == 3){
      v1.1 <- 1/r1.1[i] + 1/r1.2[i] + 1/(n.1[i]-r1.1[i]) + 1/(n.2[i]-r1.2[i])
      v1.2 <- 1/r1.1[i] + 1/r1.3[i] + 1/(n.1[i]-r1.1[i]) + 1/(n.3[i]-r1.3[i])
      v2.1 <- 1/r2.1[i] + 1/r2.2[i] + 1/(n.1[i]-r2.1[i]) + 1/(n.2[i]-r2.2[i])
      v2.2 <- 1/r2.1[i] + 1/r2.3[i] + 1/(n.1[i]-r2.1[i]) + 1/(n.3[i]-r2.3[i])
      # Covariance terms Eqn 10 from Bagos (2012)
      c1 <- 1/r1.1[i] + 1/(n.1[i] - r1.1[i])
      c2 <- 1/r2.1[i] + 1/(n.1[i] - r2.1[i])
      V.diag[[i]] <- matrix(c(v1.1,c1,0,0,
                              c1,v1.2,0,0,
                              0,0,v2.1,c2,
                              0,0,c2,v2.2),nrow=4, byrow=T)
    }
  }
}
detach(dat)
V <- bdiag(V.diag)


# Reshape the data to have one row per contrast per study
dat2 <- reshape(dat, varying=c("lor1.1","lor2.1","b.1","k.1","lor1.2","lor2.2","b.2","k.2"), 
                timevar="arm", idvar="studyID", direction="long")
dat2 <- reshape(dat2, varying=c("lor1","lor2"), 
                timevar="timept", direction="long", sep="")
rownames(dat2) <- paste0(dat2$studyID,".",dat2$timept,".",dat2$arm)

# Sort data by study and contrast, removing NA rows
dat2 <- dat2[order(dat2$studyID, dat2$timept, dat2$arm, dat2$lor, na.last=NA),]


# Construct the design matrix, with a row for each contrast and K-1 columns (parameters)
K <- length(unique(c(dat2$b,dat2$k))) # number of treatments
N <- nrow(dat2) # number of data points
X <- matrix(0, nrow=N, ncol=K-1)
attach(dat2)
for (i in 1:N){
  X[i,k[i]-1] <- 1
  if (b[i] != 1){
    X[i,b[i]-1] <- -1
  }
}
detach(dat2)

#### STEP 1 ####
# Now we can perform thresholding at the study level
thresh <- nma_thresh(X, d.mean, V, post.cov, opt.max=TRUE)


# Add in the studyID, comparator/baseline, timepoint for each study contrast
studyID <- dat2$studyID
baseline <- dat2$b
comparator <- dat2$k
timept <- dat2$timept
thresh <- cbind(studyID, timept, baseline, comparator, thresh)
print(thresh)

# Display using a forest plot, along with 95% confidence intervals for LORs
# Labels
thresh$lab <- with(thresh, sprintf("%d (%d): %d vs. %d", studyID, timept, comparator, baseline))

# Calculate approximate 95% CIs
thresh$lor <- dat2$lor
thresh$CI2.5 <- dat2$lor + qnorm(0.025)*sqrt(diag(V))
thresh$CI97.5 <- dat2$lor + qnorm(0.975)*sqrt(diag(V))

# Plot
## COLOUR
pdf("UI thresholds - study level.pdf", width=13, height=8)
with(thresh,
     thresh_forest(means=lor, t.lo=lo, t.hi=hi, kstar.lo=lo.newkstar, kstar.hi=hi.newkstar,
                   CI.lo=CI2.5, CI.hi=CI97.5, CI.title="95% Confidence Interval",
                   mean.title="LOR", label=lab, label.title="Study (Timepoint)",
                   xlab="Log Odds Ratio", xlim=c(-2.5,1), refline=0, digits=2)
)
dev.off()

## B&W
pdf("UI thresholds - study level BW.pdf", width=13, height=8)
with(thresh,
     thresh_forest(means=lor, t.lo=lo, t.hi=hi, kstar.lo=lo.newkstar, kstar.hi=hi.newkstar,
                   CI.lo=CI2.5, CI.hi=CI97.5, CI.title="95% Confidence Interval",
                   mean.title="LOR", label=lab, label.title="Study (Timepoint)",
                   xlab="Log Odds Ratio", xlim=c(-2.5,1), refline=0, digits=2,
                   greyscale=TRUE)
)
dev.off()
