############################################################
# NG25 Tocolytics EGA: Study Level Threshold Analysis
#   - Estimated Gestational Age at delivery
############################################################

source("../../../Functions/nma_threshRE.R")
source("../../../Functions/thresh_forest.R")

#### STEP 0 ####
# Read CODA data
source("./Tocolytics - read CODA.R")

# Read study data
dat.raww <- read.table(file = "../WinBUGS/study data.txt", header = TRUE, sep="\t")

n <- nrow(dat.raww)   # Number of studies

# Turn wide study data into long with one row for each arm
dat.raw <- reshape(dat.raww, varying=c("t.1","y.1","se.1",
                                       "t.2","y.2","se.2",
                                       "t.3","y.3","se.3"), 
                   timevar="arm", idvar="studyID", direction="long")

# Sort data by study and contrast, removing NA rows
dat.raw <- dat.raw[order(dat.raw$studyID, dat.raw$arm, dat.raw$y, na.last=NA),]

K <- length(unique(dat.raw$t))    # Number of treatments
N <- nrow(dat.raw)    # Number of data points

# Add in 95% CIs
dat.raw$CI2.5 <- with(dat.raw, y + qnorm(0.025)*se)
dat.raw$CI97.5 <- with(dat.raw, y + qnorm(0.975)*se)

# Study labels
dat.raw$lab <- with(dat.raw, paste0(studyID," (",t,")"))

# Create likelihood covariance matrix
V <- diag(dat.raw$se^2)

# Design matrix for study baseline parameters mu
M <- matrix(0, nrow=N, ncol=n)
M[cbind(1:N, dat.raw$studyID)] <- 1

# Random effect delta parameter design matrix L. Baseline rows from arm level
# data don't have a delta, all rows from trial level data have a delta.
L <- matrix(0, nrow=N, ncol=N-n)
j <- 1
for (i in 1:N) {
  if (dat.raw$arm[i] > 1) {
    L[i,j] <- 1
    j<-j+1
  }
}

##### STEP 1 #####
# Derive thresholds
thresh <- nma_threshRE(post.mean, V, post.cov.ddelta, mu.design=M, delta.design=L, opt.max=T)

# Merge with data
thresh <- cbind(dat.raw, thresh)

# Create threshold plots
## COLOUR
pdf("Tocolytics thresholds - study level.pdf", width=13, height=30)
with(thresh,
     thresh_forest(means=y, t.lo=lo, t.hi=hi, kstar.lo=lo.newkstar, kstar.hi=hi.newkstar,
                   CI.lo=CI2.5, CI.hi=CI97.5, label=lab, label.title="Study (Trt.)",
                   xlab="EGA (weeks)", xlim=c(25,40), mean.title="Study Estimate",
                   CI.title="95% Confidence Interval", cutoff=c(0,60), digits=2, xaxp=c(25,40,15))
)
dev.off()

# Only show studies that actually estimate a treatment effect (ie. exclude
# studies of one drug only)

excl <- vector(length=n)
attach(dat.raww)
for (i in 1:n){
  if(n.arms[i] == 2) excl[i] <- t.2[i]==t.1[i]
  else if(n.arms[i] == 3) excl[i] <- t.2[i]==t.1[i] && t.3[i]==t.1[i]
}
detach(dat.raww)

thresh.filter <- thresh[!thresh$studyID %in% which(excl),]

## COLOUR
pdf("Tocolytics thresholds - study level - filtered.pdf", width=13, height=30)
with(thresh.filter,
     thresh_forest(means=y, t.lo=lo, t.hi=hi, kstar.lo=lo.newkstar, kstar.hi=hi.newkstar,
                   CI.lo=CI2.5, CI.hi=CI97.5, label=lab, label.title="Study (Trt.)",
                   xlab="EGA (weeks)", xlim=c(25,40), mean.title="Study Estimate",
                   CI.title="95% Confidence Interval", cutoff=c(0,60), digits=2, xaxp=c(25,40,15))
)
dev.off()


# Only show studies with a threshold smaller than 10 weeks
cutoff <- 10
thresh.cut <- thresh[thresh$lo > -cutoff | thresh$hi < cutoff,]
nrow(thresh.cut)

## COLOUR
pdf("Tocolytics thresholds - study level - cutoff.pdf", width=13, height=12)
with(thresh.cut,
     thresh_forest(means=y, t.lo=lo, t.hi=hi, kstar.lo=lo.newkstar, kstar.hi=hi.newkstar,
                   CI.lo=CI2.5, CI.hi=CI97.5, label=lab, label.title="Study (Trt.)",
                   xlab="EGA (weeks)", xlim=c(25,45), mean.title="Study Estimate",
                   CI.title="95% Confidence Interval", cutoff=c(0,60), digits=2, xaxp=c(25,45,20))
)
dev.off()

