#########################################################################################
# FUNCTION: nma_thresh                           
# Find thresholds for changing decision due to bias adjustment, and what the new
# decision would be.
#
# ARGUMENTS:
#  X          = Design matrix.
#  mean.dk    = Posterior means of basic treatment parameters d_k
#  lhood      = Likelihood (data) covariance matrix.
#  post       = Posterior covariance matrix of all parameters (including
#               nuisance study means if study level).
#  opt.max    = Should the optimal decision be the maximal treatment effect
#               (TRUE, default) or the minimum (FALSE).
#  trt.rank   = Rank of the treatment to derive thresholds for. Defaults to 1,
#               thresholds for the optimum treatment.
#  trt.code   = Treatment codings of the reference trt and in the parameter
#               vector d_k. Use if some are missing (perhaps excluded) or re-ordered.
#               Default is equivalent to 1:K.
#  trt.sub    = Only look at thresholds in this subset of treatments in trt.code,
#               e.g. if some are excluded from the ranking. Default is equivalent to 1:K.
#  verbose    = Print intermediate matrices? Default is FALSE.
#
# AUTHOR: David M. Phillippo, 
#         University of Bristol
#########################################################################################

nma_thresh <- function(X, mean.dk, lhood, post,
                       opt.max=TRUE, trt.rank=1, trt.code=NULL, trt.sub=NULL,
                       verbose=FALSE)
{

  #### BASIC PARAMETER CHECKS ####
  
  # Check and print the number of treatments K.
  K <- length(mean.dk)+1
  cat("Contrasts for K =",K,"treatments.\n")
  
  # Check and print the number of nuisance parameters mu.
  m <- dim(X)[2] - (K-1)
  if (m<0) {
    stop("Error in design matrix, not enough columns for the treatments.\n")
  }
  else{
    cat("Number of extra parameters is m =",m,"in design matrix.\n")
  }
  
  # Check dimensions for lhood and post covariance matrices
  if (!all(dim(t(X) %*% lhood %*% X) == m+K-1)){
    stop("Dimensions of lhood covariance matrix do not match, should be m+K-1.\n")
  }
  if (!all(dim(post)== m+K-1)){
    stop("Dimensions of post covariance matrix do not match, should be m+K-1.\n")
  }
  
  # Treatment rank
  if (length(trt.rank)>1 | trt.rank != round(trt.rank)) stop("trt.rank should be a single integer")
  else if (trt.rank < 1 | trt.rank > K) stop("trt.rank should be between 1 and K (number of trts)")
  
  # Note about recoded treatments
  if (is.null(trt.code)){
    trt.code <- 1:K
  }
  else if(length(trt.code)!=K) stop("trt.code should be of length K.")
  else{
    cat("Using recoded treatments. Reference treatment is ",trt.code[1],
        ". Parameter vector is:\n",sep="")
    cat("\t",paste0("d[",trt.code[-1],"]",collapse=", "),"\n")
  }
  
  # Treatment subset
  if (is.null(trt.sub)){
    trt.sub <- trt.code
  }
  else if(length(trt.sub)>K) stop("Length of trt.sub should be <= K.")
  else{
    cat("Deriving thresholds on a subset of treatments:\n")
    cat("\t",trt.sub,"\n")
  }
  
  trt.sub.internal <- which(trt.code %in% trt.sub)
  
  # Error if trt.rank > length(trt.sub)
  if(trt.rank > length(trt.sub)) stop("trt.rank is larger than the length of trt.sub")
  
  ################################
    
  # Create contrast "design" matrix
  D <- matrix(nrow=K*(K-1)/2, ncol=K-1)
  j <- 1
  for (i in 1:(K-1)){
    if (i==1) D[j:(j+K-i-1),] <- diag(nrow=K-i)
    else if (i==2) D[j:(j+K-i-1),] <- cbind(rep.int(-1,K-i),diag(nrow=K-i))
    else D[j:(j+K-i-1),] <- cbind(matrix(rep.int(0,(K-i)*(i-2)),ncol=i-2),rep.int(-1,K-i),diag(nrow=K-i))
    j <- j+K-i
  }
  
  if(verbose) {
    cat("Contrast design matrix:\n")
    print(D)
  }
  
  # Create vector of contrasts d_ab
  contr <- as.vector(D %*% mean.dk)

  # Creat matrix of thresholds
  threshmat <- sweep(1 / (D %*% (post %*% crossprod(X,solve(lhood)))),
                     1, -contr, "*")
  
  if(verbose){
    cat("Influence matrix:\n")
    print(post %*% crossprod(X,solve(lhood)))
    
    cat("Threshold matrix:\n")
    print(threshmat)
  }

  # Now we only need to look at contrasts involving the optimal treatment k* 
  mean.dk.subNA <- mean.dk
  mean.dk.subNA[!(1:(K-1) %in% (trt.sub.internal-1))] <- NA
  
  if (opt.max){
    kstar <- order(c(0,mean.dk.subNA), decreasing=TRUE)[trt.rank]
  }
  else if (!opt.max){
    kstar <- order(c(0,mean.dk.subNA), decreasing=FALSE)[trt.rank]
  }
  
  if (trt.rank==1) {
    cat("Current optimal treatment is k* = ",trt.code[kstar],".\n",sep="")
  } else {
    cat("Current rank ",trt.rank," treatment is k = ",trt.code[kstar],".\n",sep="")
  }
  
  # And these contrasts have non-zero elements in the contrast design matrix D
  if (kstar > 1) {
    contr.kstar <- which(D[,kstar-1] != 0)
  }
  else {
    contr.kstar <- 1:(K-1)
  }
  
  # So we look in the corresponding rows of the threshold matrix
  threshmat.kstar <- threshmat[contr.kstar,]
  
  if(verbose){
    cat("Threshold matrix for k* related contrasts:\n")
    print(threshmat.kstar)
  }
  
  # Return the positive and negative thresholds for each observation
  # Define a function to do this which we can apply over the columns (observations)
  get.int <- function(x){
    if(all(is.infinite(x))){
      hi <- Inf
      lo <- -Inf
      hi.newkstar <- lo.newkstar <- NA
    }
    else if (all(x[!is.infinite(x)]>0)){
      hi <- min(x[!is.infinite(x)])
      hi.newkstar <- trt.code[trt.sub.internal[which(x==hi) + 
                                        (which(x==hi) >= which(trt.sub.internal==kstar))*1]]
      lo <- -Inf
      lo.newkstar <- NA
    }
    else if (all(x[!is.infinite(x)]<0)){
      hi <- Inf
      hi.newkstar <- NA
      lo <- max(x[!is.infinite(x)])
      lo.newkstar <- trt.code[trt.sub.internal[which(x==lo) + 
                                        (which(x==lo) >= which(trt.sub.internal==kstar))*1]]
    }
    else {
      hi <- min(x[x>0 & !is.infinite(x)])
      hi.newkstar <- trt.code[trt.sub.internal[which(x==hi) + 
                                        (which(x==hi) >= which(trt.sub.internal==kstar))*1]]
      lo <- max(x[x<0 & !is.infinite(x)])
      lo.newkstar <- trt.code[trt.sub.internal[which(x==lo) + 
                                        (which(x==lo) >= which(trt.sub.internal==kstar))*1]]
    }
    return(data.frame(lo=lo,lo.newkstar=lo.newkstar,hi=hi,hi.newkstar=hi.newkstar))
  }
  
  # Only look in rows which correspont to contrasts with treatments in trt.sub
  contr.trt.sub <- trt.sub.internal[-which(trt.sub.internal==kstar)] - 
    (trt.sub.internal[-which(trt.sub.internal==kstar)] >= kstar)*1
  
  # Fix for when using trt.sub to only pick out two treatments, threshmat.kstar is then only
  # one row which apply doesn't like
  if(grepl("matrix",class(threshmat.kstar[contr.trt.sub,]),ignore.case=T)){
    return(as.data.frame(do.call(rbind, apply(threshmat.kstar[contr.trt.sub,], 2, get.int))))
  }else{
    return(as.data.frame(do.call(rbind, apply(t(as.matrix(threshmat.kstar[contr.trt.sub,])), 2, get.int))))
  }
}
