#######################################################
# FUNCTION: nma_thresh2d                           
# Find and plot 2d invariant regions for a choice of
# two studies/contrasts 
#
# ARGUMENTS:
#  studies    = Indices of studies to compute invariant
#               region for. Length two vector.
#  X          = Design matrix.
#  contrasts  = Vector of posterior expected values
#               of the contrasts d_ab.
#  lhood      = Likelihood (data) covariance matrix.
#  post       = Posterior covariance matrix of all
#               parameters (including nuissance study
#               means if study level).
#  opt.max    = Should the optimal decision be the
#               maximal tretment effect (TRUE, default)
#               or the minimum (FALSE).
#  xlim       = X-axis limits, default c(-10,10).
#  ylim       = Y-axis limits, default c(-10,10).
#  xlab       = X-axis label, default "Study A"
#  ylab       = Y-axis label, default "Study B"
#  ...        = Additional graphical parameters.
#
# AUTHOR: David M. Phillippo, 
#         University of Bristol
#######################################################

require(Hmisc)

nma_thresh2d <- function(studies, X, contrasts, lhood, post, opt.max=TRUE
                        ,xlim=c(-10,10), ylim=c(-10,10)
                        ,xlab="Study A", ylab="Study B", ...){
  
  #### BASIC PARAMETER CHECKS ####
  
  # Check studies parameter is length 2 vector
  if (length(studies)!=2){
    stop("Parameter studies should be a vector of length 2.\n")
  }
  
  # Check and print the number of treatments K.
  # Note that the length of the contrast vector is K*(K-1)/2
  K <- max(Re(polyroot(c(-2*length(contrasts),-1,1))))
  if (K != round(K) || length(contrasts)==1) {
    stop("Contrast vector is not a suitable length, check it contains all contrasts.\n")
  }
  else{
    K <- as.integer(K)
    cat("Contrasts for K =",K,"treatments.\n")
  }
  
  # Check and print the number of nuisance parameters mu.
  m <- dim(X)[2] - (K-1)
  if (m<0) {
    stop("Error in design matrix, not enough columns for the treatments.\n")
  }
  else{
    cat("Number of extra parameters is m =",m,"in design matrix.\n")
  }
  
  # Check dimensions for lhood and post covariance matrices
  if (!all(dim(crossprod(X,lhood) %*% X) == m+K-1)){
    stop("Dimensions of lhood covariance matrix do not match, should be m+K-1.\n")
  }
  if (!all(dim(post)== m+K-1)){
    stop("Dimensions of post covariance matrix do not match, should be m+K-1.\n")
  }
  
  ################################
  
  # Create contrast "design" matrix
  D <- matrix(nrow=K*(K-1)/2, ncol=K-1)
  j <- 1
  for (i in 1:(K-1)){
    if (i==1) D[j:(j+K-i-1),] <- diag(nrow=K-i)
    else if (i==2) D[j:(j+K-i-1),] <- cbind(rep.int(-1,K-i),diag(nrow=K-i))
    else D[j:(j+K-i-1),] <- cbind(matrix(rep.int(0,(K-i)*(i-2)),ncol=i-2),rep.int(-1,K-i),diag(nrow=K-i))
    j <- j+K-i
  }
  
  # Get coefficients of threshold lines
  coeffs <- D %*% post %*% crossprod(X, solve(lhood)[,studies])
  
  # Now we only need to look at contrasts involving the optimal treatment k*
  if (opt.max){
    kstar <- which(c(0,contrasts[1:(K-1)])==max(0,contrasts[1:(K-1)]))
  }
  else if (!opt.max){
    kstar <- which(c(0,contrasts[1:(K-1)])==min(0,contrasts[1:(K-1)]))
  }
  cat("Current optimal treatment is k* = ",kstar,".\n",sep="")
  
  # And these contrasts have non-zero elements in the contrast design matrix D in the k* column
  contr.kstar <- which(D[,kstar-1] != 0)
  
  # So we look in the corresponding rows of the matrix coeffs
  coeffs.kstar <- coeffs[contr.kstar,]
  
  # The new optimal treatment will be the other treatment in the contrast which changes sign
  new.kstar <- 1:(K-1) + (1:(K-1) >= kstar)*1
  
  # Data frame of coefficients of threshold lines and new k* at threshold
  linedat <- data.frame(coef.c = contrasts[contr.kstar]
                        ,coef.study1 = coeffs.kstar[,1]
                        ,coef.study2 = coeffs.kstar[,2]
                        ,new.kstar = new.kstar)
  
  # Create plot
  # Threshold equation is 0 = c + ax + by, so the intercepts of lines are -c/b and gradients are -a/b
  plot(NA, xlim=xlim, ylim=ylim, xlab=xlab, ylab=ylab, ...)
  abline(h=0, lty=2, col="darkgrey")
  abline(v=0, lty=2, col="darkgrey")

  # Draw lines
  xydat <- rep(list(list()),nrow(linedat))
  for(i in 1:nrow(linedat)){
    with(linedat[i,], 
         labcurve(list(curve(-coef.study1/coef.study2 * x -coef.c/coef.study2, 
                             from=xlim[1]*2, to=xlim[2]*2, add=T, xpd=F)),
                  label=new.kstar)
         )
  }
  
  return(linedat)

}
