#######################################################
# FUNCTION: recon_vcov                         
# Reconstruct likelihood covariance matrix from 
# prior and posterior covariance matrices in the 
# contrast-level case.
#
# ARGUMENTS:
#  post       = Posterior covariance matrix.
#  prior.prec = Prior precision. Defaults to .0001
#               which is a common flat prior for NMA.
#               Not used if prior.vcov is specified.
#  prior.vcov = Prior covariance matrix. Defaults to 
#               a diagonal matrix of the same size as
#               post, with elements 1/prior.prec
#  X          = Contrast design matrix. If omitted a
#               complete network is assumed.
#  verbose    = Print intermediate matrices? Defaults
#               to FALSE.
#
# AUTHOR: David M. Phillippo, 
#         University of Bristol
#######################################################

recon_vcov <- function(post, prior.prec=.0001, prior.vcov=diag(1/prior.prec,dim(post)[1]), 
                       X=NULL, verbose=FALSE){
  
  # Prior and posterior precision matrices
  prior.P <- solve(prior.vcov)
  post.P <- solve(post)
  
  # X^T V^-1 X is equal to the difference of these precision matrices
  D <- post.P - prior.P
  
  #### COMPLETE NETWORK CASE, SOLVE EXACTLY ####
  if(is.null(X)){
    # Match terms row by row to get likelihood precisions
    # Off-diagonal elements are simply switched sign of D elements
    M1 <- -D + diag(diag(D))
    
    # Diagonal elements are D diagonal minus sum of other row elements
    M2 <- diag(diag(D) - rowSums(M1))
    
    # M is the matrix of p elements
    M <- M1 + M2
    
    # Now read off the elements
    
    # Diagonal elements are the p_1b
    p1 <- diag(M)
    
    # Upper triangular rows are
    #    p_23 ......... p_2K
    #         p_34 .... p_3K
    #              .........
    #                   p_(K-1)K
    
    # R reads down columns, so use lower.tri instead (symmetric matrix) to get
    # the remaining parameters in the right order.
    p2 <- M[lower.tri(M)]
    
    # The likelihood precision matrix is then
    lik.cov <- diag(1/c(p1,p2))
    return(lik.cov)
  }
  
  #### INCOMPLETE NETWORK CASE, FIND OPTIMUM SOLUTION WITH NNLS ####
  # Find NNLS solution to the equation Rp=s
  else{
    
    # Basic parameter check
    if(dim(post)[1] != dim(X)[2]) {
      stop("Number of parameters in design matrix does not match posterior covariance matrix.\n")
    }
    
    # Number of treatments
    K <- dim(post)[1]+1
    
    require(nnls)
    
    # Construct s vector from difference in precision matrices
    s <- D[lower.tri(D, diag=TRUE)]
    
    # Construct R matrix from the design matrix X
    R <- matrix(0, nrow=K*(K-1)/2,ncol=K*(K-1)/2)
    
    # Loop through rows of X
    for (i in 1:nrow(X)){
      a.ind <- ifelse(any(X[i,]==-1),which(X[i,]==-1),0)
      b.ind <- ifelse(any(X[i,]==1),which(X[i,]==1),0)

      # Edges ab
      if (a.ind>0 && b.ind>0) {
        R[1+K*(K-1)/2-(K-a.ind+1)*(K-a.ind)/2, 
          1+K*(K-1)/2-(K-a.ind)*(K-a.ind-1)/2+b.ind-a.ind-1] <- 1
        R[1+K*(K-1)/2-(K-b.ind+1)*(K-b.ind)/2, 
          1+K*(K-1)/2-(K-a.ind)*(K-a.ind-1)/2+b.ind-a.ind-1] <- 1
        
        R[1+K*(K-1)/2-(K-a.ind+1)*(K-a.ind)/2+b.ind-a.ind, 
          1+K*(K-1)/2-(K-a.ind)*(K-a.ind-1)/2+b.ind-a.ind-1] <- -1
        
      }
      # Edges 1b
      else if(a.ind==0 && b.ind>0){
        R[1+K*(K-1)/2-(K-b.ind+1)*(K-b.ind)/2, b.ind] <- 1
      }
    }
    
    # Drop parameters with no edges (ie remove columns with all zeros)
    R <- R[,!apply(R,2,identical,rep(0,K*(K-1)/2))]
    
    if(verbose) {
      cat("Design matrix R for NNLS system Rp=q is:\n")
      print(R)
    }
    
    # Solve using NNLS to obtain likelihood precisions
    sol <- nnls(R,s)
    cat("Likelihood precisions found using NNLS.\n")
    cat("Residual Sum of Squares:",format(sol$deviance,digits=6),"\n")
    cat("             --------------------      \n")
    
    # RSS from elements which are fixed (i.e. trying to equate posterior precision with zero)
    ind.R0 <- ifelse(any(apply(R,1,function(x) all(x==0)))
                     ,which(apply(R,1,function(x) all(x==0)))
                     ,NA)
    rss0 <- ifelse(is.na(ind.R0),0,sum((R%*%sol$x - s)[ind.R0]^2))
    cat("              RSS fixed:",format(rss0,digits=6),"\n")
    
    # RSS from elements which are actually fitted
    rss1 <- ifelse(is.na(ind.R0),sum((R%*%sol$x - s)^2),sum((R%*%sol$x - s)[-ind.R0]^2))
    cat("             RSS fitted:",format(rss1,digits=6),"\n")
    cat("             --------------------      \n")
    
    lik.cov <- diag(1/sol$x)
    
    # Check whether any variances calculated are infinite
    inftf <- is.infinite(diag(lik.cov))
    if(any(inftf)){
      warning("Returned some infinite variances. These will be not be included in KL calculation.")
    }
    
    # Kullback-Leibler Divergence of estimated posterior (arising from fitted
    # likelihood with infinite variances removed) from the "true" posterior
    
    # The posterior cov from the fitted likelihood cov
    post.cov.fit <- solve(solve(prior.vcov) + t(X[!inftf,]) %*% 
                                              solve(lik.cov[!inftf,!inftf]) %*% 
                                              X[!inftf,])
    
    # KL divergence of this from "true" post cov
    KL.div <- 1/2 * (sum(diag(solve(post.cov.fit) %*% post)) - dim(post)[1] 
                     + log(det(post.cov.fit)/det(post)))
    cat("Kullback-Leibler Divergence of fitted from 'true' posterior:",KL.div,"\n")
    
    return(lik.cov)
  }
}