########################################################
# FUNCTION: thresh_forest                         
# Display thresholds on a forest plot, along with the
# means and optionally credible intervals.
#
# ARGUMENTS:
#  means        = Study estimates or posterior
#                 means of contrasts
#  t.lo         = Negative threshold values
#  t.hi         = Positive threshold values
#  kstar.lo     = New k* optimal treatments for
#                 negative threshold
#  kstar.hi     = New k* optimal treatments for
#                 positive threshold
#  CI.lo        = Lower values of CIs (optional)
#  CI.hi        = Upper values of CIs (optional)
#  CI.title     = Title for CI column, default 
#                 "95% Credible Interval"
#  label        = Row labels
#  label.title  = Column title for labels (e.g.
#                 "Contrast" or "Study")
#  mean.title   = Column title for means, default
#                 "Posterior Mean"
#  xlab         = Label for x-axis
#  xlim         = View limits for x-axis
#  xlim.p       = Instead of setting xlim explicitly,
#                 this tuning parameter in (0,1]
#                 sets the view to display roughly a 
#                 proportion of the invariant
#                 intervals. Default 0.75.
#  main         = Main title for plot (optional)
#  sigfig       = Significant figures to display in the
#                 table. Default is 3.
#  digits       = Decimal places to display in the table.
#                 Default is NULL, displaying using
#                 sigfig instead. If set overrides sigfig.
#  refline      = X position for reference line, or NULL
#                 to not show line. Default NULL.
#  greyscale    = Use greyscale colour palette? Default
#                 is FALSE, use vibrant full colour.
#  clinsig      = Set the clinical significance level.
#                 Mark rows with a dagger that have 
#                 thresholds less than this. Default
#                 NULL (none marked).
#  cutoff       = A single value or vector pair.
#                 Thresholds larger than this value or
#                 outside the interval pair will
#                 be cut off and display NT. Default
#                 is NULL (no cut off).
#  ...          = Additional parameters to pass to
#                 plotting functions. May break things.
#
# AUTHOR: David M. Phillippo, 
#         University of Bristol
########################################################

require(shape)

thresh_forest <- function(means, t.lo, t.hi, kstar.lo, kstar.hi, CI.lo=NULL, CI.hi=NULL, 
                          CI.title="95% Credible Interval", label, label.title, 
                          mean.title="Posterior Mean", xlab, xlim=NULL, xlim.p=0.25, main=NULL, 
                          sigfig=3, digits=NULL, refline=NULL, greyscale=FALSE, clinsig=NULL, 
                          cutoff=NULL, ...){
  
  # Set graphical parameters
  gp <- par(mfrow=c(1,2), xpd=NA, lend=1, xaxs="i", yaxs="i")
  
  # Define colour palettes
  if(greyscale){
    palette(c(rgb(.8,.8,.8),  # light grey for wide intervals
              rgb(.4,.4,.4)   # dark grey for narrow intervals
    ))
  }
  else{
    palette(c(rgb(183,203,237,maxColorValue = 255), # blue for wide intervals
              rgb(237,183,203,maxColorValue = 255)  # red for narrow intervals
    ))
  }
  
  # Reverse plotting coordinates on y-axis
  yax <- length(t.lo):1
  
  # Are posterior credible intervals requested?
  credint <- !is.null(CI.lo) && !is.null(CI.hi)
  
  # Centre invariant intervals around the means
  t.lo.c <- t.lo + means
  t.hi.c <- t.hi + means
  
  # Calculate which invariant intervals are narrower than the CIs
  t.lo.narrow <- rep(1,length(t.lo))
  t.lo.narrow[t.lo.c > CI.lo] <- 2
  
  t.hi.narrow <- rep(1,length(t.hi))
  t.hi.narrow[t.hi.c < CI.hi] <- 2
  
  t.narrow <- apply(cbind(t.lo.narrow,t.hi.narrow),1,max)
  
  # Use significant figures or decimal places?
  sf <- is.null(digits)
  
  ##### DATA TABLE #####
  
  # Blank plot to set up the plot window
  gp2 <- par(mar=c(5, 1, 2, 2) + 0.1)
  plot(means, yax, xlim=c(0,ifelse(credint,5,3)), ylim=c(0,length(yax)+1),
       pch=NA, xaxt="n", yaxt="n", xlab="", ylab="", bty="n", ...)
  
  # Headers
  if (credint){
    # left justified row label header
    axis(3, at=c(-.1,0,1.5), lwd.ticks=0, padj=1, hadj=0, labels=c(NA,label.title,NA))
    # other headers centred
    axis(3, at=c(1.5,3,5,6), lwd.ticks=0, hadj=0.5, padj=c(1,1,0.8,1),
         labels=c(mean.title,
                  CI.title,
				  expression(paste("Invariant Interval with ",tilde(k),"*")),
                  NA))
  }
  else{
    axis(3, at=c(-.1,0,1.5), lwd.ticks=0, padj=1, hadj=0, labels=c(NA,label.title,NA))
    axis(3, at=c(1.5,3,5), lwd.ticks=0, hadj=0.5, padj=c(1,0.8,1),
         labels=c(mean.title,
				  expression(paste("Invariant Interval with ",tilde(k),"*")),
                  NA))
  }
  
  ### Row names
  
  # Sometimes labels come in as factors, that doesn't work so turn them back
  # into character vectors.
  if(is.factor(label)) label <- as.vector(label)
  
  # First, mark rows with thresholds less than clinsig with a dagger
  if (!is.null(clinsig)){
    which.ltclinsig <- which(apply(abs(cbind(t.lo, t.hi)), 1, min, na.rm=T) < 0.5)
    if (length(which.ltclinsig) > 0){
      if (!is.expression(label)) label <- as.expression(label)
      for (i in 1:length(label)){
        if (i %in% which.ltclinsig){
          label[i] <- as.expression(substitute(""^"\206"*xx,list(xx=eval(label[i]))))
        }else{
          label[i] <- as.expression(substitute(" "*xx,list(xx=eval(label[i]))))
        }
      }
    }
  }
  
  # Then make short interval row labels bold 
  # If they are expressions...
  if (is.expression(label)){
    labbold <- as.expression(label)
    for (i in 1:length(label)){
      boldme <- labbold[i]
      if (t.narrow[i]==2) labbold[i] <- as.expression(substitute(bold(boldme), list(boldme=boldme[[1]])))
    }
  }
  # If labels are just strings then font=2 will make them bold
  else labbold <- label
  
  # Plot row labels
  text(0, yax, labels=labbold, adj=c(0,0.5), font=t.narrow)
  
  #############
  
  # Set up cutoff
  if (is.null(cutoff)) cutoff <- c(-Inf,Inf)
  else if (length(cutoff)<=1) cutoff <- c(-abs(cutoff),abs(cutoff))
  
  # Function to print with significant digits or decimal places
  printsig <- function(num, dig=ifelse(sf,sigfig,digits), cutyn=FALSE){
    if(cutyn && (num >= min(Inf,cutoff[2]) || num <= max(-Inf,cutoff[1]))) "NT"
    else if(sf) gsub("\\.$","",formatC(signif(num,digits=dig), digits=dig, format="fg", flag="#"))
    else formatC(num, dig, format="f")
  }
  
  # Means
  text(1.5, yax, labels=printsig(means), adj=c(0.5,0.5))
  
  # Posterior CIs
  if (credint){
    CI.text <- rep("",length(CI.lo))
    for (i in 1:length(CI.lo)){
      CI.text[i] <- paste0("(",printsig(CI.lo[i]),", ",
                           printsig(CI.hi[i]),")")
    }
    text(3, yax, labels=CI.text, adj=c(0.5,0.5))
  }
  
  # Invariant intervals
  Int.text <- rep("",length(t.lo))
  for (i in 1:length(t.lo)){
    Int.text[i] <- paste0("(",printsig(t.lo.c[i], cutyn=T),", ",
                         printsig(t.hi.c[i], cutyn=T),")")
  }
  text(ifelse(credint,5,3), yax, labels=Int.text, adj=c(0.5,0.5))
  
  # New k*
  klo.text <- format(kstar.lo,digits=0)
  klo.text[is.na(kstar.lo) | t.lo.c <= cutoff[1]] <- "\u2013"
    text(ifelse(credint,4.3,2.3), yax, labels=klo.text, adj=c(0.5,0.5))
  
  khi.text <- format(kstar.hi,digits=0)
  khi.text[is.na(kstar.hi) | t.hi.c >= cutoff[2]] <- "\u2013"
  text(ifelse(credint,5.7,3.7), yax, labels=khi.text, adj=c(0.5,0.5))
  
  par(gp2)
  

  ##### FOREST PLOT #####
  
  # Calculate xlim if not set
  if (is.null(xlim)){
    xlim <- quantile(c(t.lo.c,t.hi.c),c((1-xlim.p)/2,1-(1-xlim.p)/2),na.rm=TRUE)
  }  
  
  # Fake plot the means to set up the plot window
  gp2 <- par(mar=c(5, 5, 2, 2) + 0.1)
  plot(means, yax, main=main, ylim=c(0,length(yax)+1),
       xlab=xlab, ylab="", yaxt="n", xlim=xlim, bty="n" ,pch=NA,...)
  
  # Plot reference line
  abline(v=refline, lty=2, col=rgb(.4,.4,.4), xpd=FALSE)
  
  # Calculate which intervals to truncate
  t.lo.whichtrunc <- which(t.lo.c < xlim[1])
  t.hi.whichtrunc <- which(t.hi.c > xlim[2])

  # Calculate truncated intervals
  t.lo.trunc <- apply(cbind(t.lo.c,xlim[1]),1,max)
  t.hi.trunc <- apply(cbind(t.hi.c,xlim[2]),1,min)
  
  # And the same for CIs, if requested
  if(credint){
    CI.lo.whichtrunc <- which(CI.lo < xlim[1])
    CI.hi.whichtrunc <- which(CI.hi > xlim[2])
    
    CI.lo.trunc <- apply(cbind(CI.lo,xlim[1]),1,max)
    CI.hi.trunc <- apply(cbind(CI.hi,xlim[2]),1,min)
  }
  
  # Plot invariant intervals
  int.lwd <- 8
  segments(y0=yax, x0=t.lo.trunc, x1=means, lwd=int.lwd, col=t.lo.narrow, lend=1)
  segments(y0=yax, x0=t.hi.trunc, x1=means, lwd=int.lwd, col=t.hi.narrow, lend=1)
  # Use lend=1 for butt line ends - round and square make intervals wider!
  
  # Add arrowheads on to truncated intervals
  # Assume pdf device for specifying widths (lwd=1 is 1/96in, and 75/600cm is 6/96in*2)
  Arrowhead(xlim[1],yax[t.lo.whichtrunc], angle=180, arr.type="triangle", arr.adj=0,
            arr.col=t.lo.narrow[t.lo.whichtrunc], lty=0, 
            arr.width=int.lwd/96*2*1.02, arr.length=1.6*(int.lwd/96*2))
  
  Arrowhead(xlim[2],yax[t.hi.whichtrunc], arr.type="triangle", arr.adj=0,
            arr.col=t.hi.narrow[t.hi.whichtrunc], lty=0, 
            arr.width=int.lwd/96*2*1.02, arr.length=1.6*(int.lwd/96*2))
  
  # Plot CIs
  if (credint){
    segments(y0=yax, x0=CI.lo.trunc, x1=CI.hi.trunc,
             lwd=1)
  
    # Add arrowheads to truncated CIs
    Arrowhead(xlim[1], yax[CI.lo.whichtrunc], arr.type="curved", 
              arr.length=.2, lty=0, angle=180)
    Arrowhead(xlim[2], yax[CI.hi.whichtrunc], arr.type="curved", 
              arr.length=.2, lty=0)
  }
  
  # Plot the means for real, on top of everything else
  points(means, yax, pch=21, col="black", bg="white")
  
  par(gp2)
  
  ##### ADD LEGEND TO OVERLAY PLOT #####
  par(fig = c(0, 1, 0, 1), oma = c(.5, 2, 0, 0), mar = c(0, 0, 0, 0), new = TRUE)
  plot(0, 0, type = "n", bty = "n", xaxt = "n", yaxt = "n")
  if (credint){
    legend("bottomleft",c("Posterior Mean",CI.title,"Invariant Interval"),
           lty=c(NA,1,1), pch=c(21,NA,NA), col=c("black","black",1), lwd=c(NA,1,8), pt.bg="white",
           horiz=TRUE, bty="n",
           text.width = max(abs(strwidth(c("Posterior Mean",CI.title,"Invariant Interval"))))+.05)
    # text.width sets space for legend labels, make these a little wider than default
  }
  else {
    legend("bottomleft",c("Posterior Mean","Invariant Interval"),
           lty=c(NA,1), pch=c(21,NA), col=c("black",1), lwd=c(NA,8), pt.bg="white",
           horiz=TRUE, bty="n")
  }
  
  # Reset default palette, graphical parameters
  palette("default")
  par(gp)
}
